/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using System.ComponentModel;


namespace xGameConsoleXNA
{
	namespace GenericCommands
	{
		
		public class FileCommands : BaseCommands
		{
			
			// Implements DXTools.IConsoleExtender, idisposable
			
			public FileCommands()
			{
			}
			
			public FileCommands(xGameConsoleXNA.Console Console) : base(Console)
			{
			}
			
			// Private WithEvents CommandEvents As DXTools.CommandManager
			//Private mobjManager As ConsoleExtenderManager
			//Private mblnIsConnected As Boolean = False
			// Private mobjCommandList As Collections.Specialized.StringCollection
			//Private mblnRemovingCommands As Boolean = False
			
			[Category("File"), Description("Displays the current directory. Use /n or -n to prevent writing to the console.")]public string GetCurrentDirectory(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				// we make a call to processargumants to allow any user specified functions to run end generate any arguments
				// someone may decide to use a function to determin weather or not to print the current dir to the console using
				// the /n or -n switches.
				args = mobjManager.ProcessArguments(args);
				if (args != null)
				{
					if (args[0].Trim().ToLower() != "/n" && args[0].Trim().ToLower() != "-n")
					{
						this.Print(System.IO.Directory.GetCurrentDirectory());
					}
				}
				else
				{
					this.Print(System.IO.Directory.GetCurrentDirectory());
				}
				return System.IO.Directory.GetCurrentDirectory();
			}
			
			[Category("File"), Description("Displays a list of files in the current directory.")]public string ListFiles(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				
				string[] Files;
				string Tmp;
				Tmp = "*.*";
				if (args != null)
				{
					if (args.Length != 0)
					{
						Tmp = args[0].ToLower().Trim();
					}
				}
				try
				{
					Files = System.IO.Directory.GetFiles(System.IO.Directory.GetCurrentDirectory(), Tmp);
				}
				catch (ArgumentException)
				{
					this.Print("Bad search pattern.");
					return null;
					
				}
				catch (Exception)
				{
					this.Print("Error occoured during file search. Check search pattern.");
					return null;
					
				}
				
				this.PrintBlankLine();
				this.Print("List of files found");
				this.Print("----------------------");
				
				foreach (string tempLoopVar_Tmp in Files)
				{
					Tmp = tempLoopVar_Tmp;
					this.Print(System.IO.Path.GetFileName(Tmp));
				}
				
				return null;
			}
			
			[Category("File"), Description("Sets the current directory.")]public string SetCurrentDirectory(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				
				try
				{
					System.IO.Directory.SetCurrentDirectory(args[0]);
					
					// if [ getexpr exp1 exp2 ]
					// getexpr returns "quit" if true
					// getexpr returns  "quit otherwise echo did not quit" if false
					
					// if true quit
					// if true quit otherwise echo did not quit
					
				}
				catch (ArgumentNullException)
				{
					this.Print("No path was specified. ");
					
				}
				catch (ArgumentException)
				{
					this.Print("path is a zero-length string, contains only white space, or contains one or more invalid characters " + System.IO.Path.GetInvalidPathChars());
					
				}
				catch (System.IO.PathTooLongException)
				{
					this.Print("The length of path or the absolute path information for path exceeds the system-defined maximum length. ");
					
				}
				catch (System.IO.FileNotFoundException)
				{
					this.Print("The specified path was not found.");
					
					
				}
				this.Print("Current directory is " + System.IO.Directory.GetCurrentDirectory());
				return null;
			}
			
			//Private Sub DoAddCommand(ByVal Name As String, ByVal CB As DXTools.CommandCallback)
			//    If mobjManager Is Nothing Then Exit Sub
			//    If mobjExtenderManager.Console Is Nothing Then Exit Sub
			//    If mobjExtenderManager.Console.CommandManager Is Nothing Then Exit Sub
			//    mobjExtenderManager.Console.CommandManager.AddCommand(Name, CB)
			//    mobjCommandList.Add(Name)
			//End Sub
			
			//Private Sub CommandIsBeingRemoved(ByVal CommandName As String, ByVal e As System.ComponentModel.CancelEventArgs)
			//    ' we can now prevent any of our commands from being removed
			//    e.Cancel = (mobjCommandList.IndexOf(CommandName) <> -1) And (mblnRemovingCommands = False)
			//    If e.Cancel Then
			//        If mblnIsConnected = False Then Exit Sub
			//        If mobjExtenderManager.Console Is Nothing Then Exit Sub
			//        mobjExtenderManager.Console.Print("FileCommands: Some process tried to remove command " & CommandName)
			//    End If
			//End Sub
			
			//Private Sub DoRemoveCommands()
			//    If mobjManager Is Nothing Then Exit Sub
			//    If mobjExtenderManager.Console Is Nothing Then Exit Sub
			//    If mobjExtenderManager.Console.CommandManager Is Nothing Then Exit Sub
			//    mblnRemovingCommands = True
			//    Dim tmp As String
			//    For Each tmp In mobjCommandList
			//        mobjExtenderManager.Console.CommandManager.RemoveCommand(tmp)
			//    Next
			//    mobjCommandList.Clear()
			//    mblnRemovingCommands = False
			//End Sub
			
			//Public ReadOnly Property IsConnected() As Boolean Implements DXTools.IConsoleExtender.IsConnected
			//    Get
			//        Return mblnIsConnected
			//    End Get
			//End Property
			
			public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
			{
				base.OnConnect(Manager, ExtenderManager);
				DoAddCommand("getfiles", new xGameConsoleXNA.CommandCallback( this.ListFiles));
				DoAddCommand("getdir", new xGameConsoleXNA.CommandCallback( this.GetCurrentDirectory));
				DoAddCommand("setdir", new xGameConsoleXNA.CommandCallback( this.SetCurrentDirectory));
			}
			
			//    Public Sub OnDisconnect() Implements DXTools.IConsoleExtender.OnDisconnect
			//        mblnIsConnected = False
			//        mobjCommandList.Clear()
			//        mobjCommandList = Nothing
			//        mobjManager = Nothing
			//        RemoveHandler mobjExtenderManager.Console.CommandManager.BeforeRemoveCommand, AddressOf CommandIsBeingRemoved
			//    End Sub
			
			
			//#Region " IDisposable Support "
			//    Private disposed As Boolean = False
			//    ' IDisposable
			//    Private Overloads Sub Dispose(ByVal disposing As Boolean)
			//        If Not Me.disposed Then
			//            If disposing Then
			//                DoRemoveCommands()
			//                If mobjCommandList Is Nothing = False Then
			//                    mobjCommandList.Clear()
			//                End If
			//                mobjCommandList = Nothing
			//                If mobjManager Is Nothing = False Then
			//                    If mobjExtenderManager.Console Is Nothing = False Then
			//                        If mobjExtenderManager.Console.CommandManager Is Nothing = False Then
			//                            RemoveHandler mobjExtenderManager.Console.CommandManager.BeforeRemoveCommand, AddressOf CommandIsBeingRemoved
			//                        End If
			//                    End If
			//                End If
			//                mobjManager = Nothing
			
			//            End If
			//            ' TODO: put code to free unmanaged resources here
			//        End If
			//        Me.disposed = True
			//    End Sub
			
			//    ' This code added by Visual Basic to correctly implement the disposable pattern.
			//    Public Overloads Sub Dispose() Implements IDisposable.Dispose
			//        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
			//        Dispose(True)
			//        GC.SuppressFinalize(Me)
			//    End Sub
			
			//    Protected Overrides Sub Finalize()
			//        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
			//        Dispose(False)
			//        MyBase.Finalize()
			//    End Sub
			//#End Region
		}
	}
	
}
