/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class GenericCommands : BaseCommands
        {


            private const string STR_ListCommands = "Displays a list of all the commands registered with the console." + "\n" + "Use /w parameter to list multiple commands on one line." + "\n" + "Specifying a numeric value will control how much spacing there is between" + "\n" + "the comamnds and there descriptions. NOTE: /w must preceed any numeric value.";

            #region " Commands ... "
            [Category("Generic"), Description("Displays a list of command categories or displays a category attribute for a command.\n" +
                                              "Example: cat >category<")]
            private string ShowCategory(string[] args)
            {
                string Ret = string.Empty;
                if (mblnIsConnected == false) return Ret;

                // if autoprocessing is not enabled then we need to process the args
                if (mobjManager.AutoProcessArguments == false)
                {
                    args = mobjManager.ProcessArguments(args);
                }

                System.Collections.Specialized.StringCollection items = new System.Collections.Specialized.StringCollection();

                if (args == null || args.Length == 0)
                {
                    string[] cats;
                    cats = this.CommandManager.GetCategoryList();
                    if (cats != null)
                    {
                        items.Add(Resources.Categories);
                        items.Add(Helpers.String('-', 12));
                        items.AddRange(cats);
                        items.Add(Resources.NumberCategories.Replace(Resources.RKEY, cats.Length.ToString()));
                    }
                }
                else if (args != null && args.Length >= 1)
                {
                    if (args[0].Trim().ToLower() != string.Empty)
                    {
                        string[] cmds;
                        cmds = this.CommandManager.GetCommandList(args[0]);

                        if (cmds != null)
                        {
                            items.Add(Resources.Commands);
                            items.Add(Helpers.String('-', 12));
                            items.AddRange(cmds);
                            items.Add(Resources.NumberCommands.Replace(Resources.RKEY, cmds.Length.ToString()));
                        }
                    }
                }

                // write text to console
                for (int idx = 0; idx <= items.Count - 1; idx++)
                {
                    this.Print(items[idx]);
                }

                // let user know if no commands found
                if (items.Count == 0) this.Print(Resources.NoCommandsFound);

                return Ret;
            }

            [Category("Generic"), Description("Loads a set of command aliases from a file.")]
            public string LoadAliasList(string[] args)
            {
                if (mblnIsConnected && (mobjManager != null))
                {
                    // if autoprocessing is not enabled then we need to process the args
                    if (mobjManager.AutoProcessArguments == false)
                    {
                        args = mobjManager.ProcessArguments(args);
                    }

                    // check if a file was specified
                    if (args == null || args.Length == 0)
                    {
                        this.Print(Resources.NoFileSpecified);
                        return null;
                    }

                    // get filename specified
                    string File;
                    File = string.Join(" ", args);

                    // check for invalid chars
                    if (File.IndexOfAny(System.IO.Path.GetInvalidFileNameChars()) != -1 || File.IndexOfAny(System.IO.Path.GetInvalidPathChars()) != -1)
                    {
                        this.Print(Resources.PathOrFileInvalidChars);
                        return null;
                    }

                    // if path is not rooted get full path
                    if (System.IO.Path.IsPathRooted(File) == false)
                    {
                        File = System.IO.Path.GetFullPath(File);
                    }

                    // check if directoty exists
                    if (System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(File)) == false)
                    {
                        this.Print(Resources.DirectoryNameNonExistant);
                        return null;
                    }

                    // check if file exists
                    if (System.IO.File.Exists(File) == false)
                    {
                        this.Print(Resources.FileNotFound);
                        return null;
                    }

                    // get alias names
                    string[] lines;
                    lines = System.IO.File.ReadAllLines(File);
                    int Count = 0;
                    foreach (string item in lines)
                    {
                        string Comm;
                        string aName;
                        int Index;
                        Comm = item;

                        // check for alias / command separation
                        Index = Comm.IndexOf(" ");
                        if (Index != -1)
                        {
                            // get alias name and command
                            aName = Comm.Substring(0, Index).Trim().ToLower();
                            Comm = Comm.Substring(Index).Trim();
                            try
                            {
                                // add alias
                                mobjManager.AddAlias(aName, Comm);
                                // write to console that alias was loaded
                                // this.Print("Alias loaded. \'" + aName + "\' executes " + Comm);
                                Count++;
                            }
                            catch (ArgumentException arge)
                            {
                                // write errormsg to console
                                this.Print(arge.Message);
                            }
                        }
                    }

                    // write to console how many aliases were loaded
                    this.Print(Resources.NumberAliasLoaded.Replace(Resources.RKEY, Count.ToString()));
                }

                return null;
            }

            [Category("Generic"), Description("Saves the current set of command aliases to a file.")]
            public string SaveAliasList(string[] args)
            {
                if (mblnIsConnected && (mobjManager != null))
                {
                    // if autoprocessing is not enabled then we need to process the args
                    if (mobjManager.AutoProcessArguments == false)
                    {
                        args = mobjManager.ProcessArguments(args);
                    }

                    // check if file was specified
                    if (args == null || args.Length == 0)
                    {
                        this.Print(Resources.NoFileSpecified);
                        return null;
                    }

                    string File;
                    File = string.Join(" ", args);
                    if (System.IO.Path.GetDirectoryName(File).IndexOfAny(System.IO.Path.GetInvalidPathChars()) != -1)
                    {
                        this.Print("Path contains invalid characters.");
                        return null;
                    }
                    if (System.IO.Path.GetFileName(File).IndexOfAny(System.IO.Path.GetInvalidFileNameChars()) != -1)
                    {
                        this.Print("File contains invalid characters.");
                        return null;
                    }
                    if (System.IO.Path.IsPathRooted(File) == false)
                    {
                        File = System.IO.Path.Combine(System.IO.Directory.GetCurrentDirectory(), File);
                    }
                    if (System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(File)) == false)
                    {
                        this.Print("Directory with that name does not exist.");
                        return null;
                    }

                    // get alias names
                    string Tmp;
                    string[] Names;
                    Names = mobjManager.GetAliasList();
                    System.IO.StreamWriter F;
                    if (System.IO.File.Exists(File))
                    {
                        System.IO.File.Delete(File);
                    }
                    F = new System.IO.StreamWriter(File);
                    foreach (string tempLoopVar_Tmp in Names)
                    {
                        Tmp = tempLoopVar_Tmp;
                        string Comm;
                        Comm = mobjManager.GetCommandFromAlias(Tmp);
                        F.WriteLine(Tmp + " " + Comm);
                    }
                    F.Flush();
                    F.Close();
                    F = null;
                }
                return null;
            }

            [Category("Generic"), Description("Displays help for a command.")]
            private string ShowHelp(string[] args)
            {
                if (mblnIsConnected && (mobjManager != null))
                {
                    // check if command or alias name specified
                    if (args == null || args.Length == 0)
                    {
                        this.Print(Resources.NoHelpArgument);
                        return null;
                    }

                    // try to retrieve specified commmand
                    CommandCallback item;
                    item = mobjManager.GetCommand(args[0], true);
                    if (item == null)
                    {
                        this.Print(Resources.SpecifiedCommandNotExist);
                    }
                    else
                    {

                        // try to get description sttribute
                        System.ComponentModel.DescriptionAttribute desc;
                        desc = Helpers.GetAttribute<System.ComponentModel.DescriptionAttribute>(item.Method);

                        // write description out to console if present
                        if (desc != null)
                        {
                            this.Print(desc.Description);
                        }
                        else
                        {
                            this.Print(Resources.NoHelpAvailible);
                        }
                    }

                }

                return null;
            }

            [Category("Generic"), Description(STR_ListCommands)]
            private string ListCommands(string[] args)
            {
                if (mblnIsConnected && (mobjManager != null))
                {

                    string[] Commands;
                    Commands = mobjManager.GetCommandList();
                    this.Print(Resources.Commands);
                    this.Print(Helpers.String('-', 12));
                    this.Print(Commands);
                    this.Print(Resources.NumberCommands.Replace(Resources.RKEY, Commands.Length.ToString()));

                    //int intMinSpacing = 25;
                    //int intSpacing = intMinSpacing;



                    //string Tmp = "";
                    //if (args != null)
                    //{
                    //    if (args.Length == 1)
                    //    {
                    //        Tmp = args[0].ToLower().Trim();
                    //        if (Tmp != "/w")
                    //        {
                    //            if (Helpers.IsNumeric(args[0]))
                    //            {
                    //                intSpacing = Math.Max(System.Convert.ToInt32(args[0]), intMinSpacing);
                    //            }
                    //        }
                    //    }
                    //    else if (args.Length == 2)
                    //    {
                    //        Tmp = args[0].ToLower().Trim();
                    //        if (Helpers.IsNumeric(args[1]))
                    //        {
                    //            intSpacing = Math.Max(System.Convert.ToInt32(args[1]), intMinSpacing);
                    //        }
                    //    }
                    //}

                    //this.PrintBlankLine();
                    //this.Print("Command" + Helpers.Space(intSpacing - 7) + "Description");
                    //this.Print("--------" + Helpers.String('-', intSpacing + 7));
                    //switch (Tmp)
                    //{
                    //    case "/w":

                    //        this.Print("\"/w\" param not yet implemented");
                    //        break;

                    //    default:

                    //        // mobjExtenderManager.Console.Print(Commands)

                    //        foreach (string tempLoopVar_Tmp in Commands)
                    //        {
                    //            Tmp = tempLoopVar_Tmp;
                    //            string cd;
                    //            cd = GetDescription(Tmp);
                    //            if (cd.IndexOf("\n") != -1)
                    //            {
                    //                //cd = cd.Replace("\r\n", Constants.vbCr);
                    //                string[] Parts;
                    //                string[] search = new string[1];
                    //                search[0] = "\n";
                    //                Parts = cd.Split(search, StringSplitOptions.RemoveEmptyEntries);
                    //                this.Print(Tmp + Helpers.Space(Math.Max(intSpacing, Tmp.Length) - Math.Min(intSpacing, Tmp.Length)) + Parts[0]);
                    //                int idx;
                    //                for (idx = 1; idx <= Parts.Length - 1; idx++)
                    //                {
                    //                    this.Print(Helpers.Space(intSpacing) + Parts[idx]);
                    //                }
                    //            }
                    //            else
                    //            {
                    //                this.Print(Tmp + Helpers.Space(Math.Max(intSpacing, Tmp.Length) - Math.Min(intSpacing, Tmp.Length)) + cd);
                    //            }
                    //        }
                    //        break;
                    //}

                    //this.PrintBlankLine();
                    //this.Print(mobjManager.CommandCount.ToString() + " command(s).");
                }

                return null;
            }

            [Category("Generic"), Description("Writes text out to the console.")]
            private string Echo(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {
                    // if autoprocessing is not enabled then we need to process the args
                    if (mobjManager.AutoProcessArguments == false)
                    {
                        args = mobjManager.ProcessArguments(args);
                    }

                   
                                this.Print(string.Join(Helpers.Space(1), args));
                       
                }

                return string.Join(" ", args);
            }

            [Category("Generic"), Description("Deletes an alias for a command.")]
            private string DeleteAlias(string[] args)
            {
                if (mblnIsConnected)
                {

                    if (args == null)
                    {
                        this.Print("Expected 1 parameter!");
                        this.Print("Example: deletealias >alias name<");
                    }
                    else
                    {
                        if (args.Length == 1)
                        {
                            if (mobjManager == null)
                            {
                                this.Print("No command manager is assigned to the console!");
                            }
                            else
                            {
                                try
                                {
                                    mobjManager.RemoveAlias(args[0]);
                                }
                                catch (ArgumentException arg)
                                {
                                    this.Print(arg.Message);
                                    return null;
                                }
                                this.Print("Alias removed.");
                            }
                        }
                        else
                        {
                            this.Print("Expected 1 parameter!");
                            this.Print("Example: deletealias >alias name<");
                        }
                    }
                }
                return null;
            }

            [Category("Generic"), Description("Creates an alias for a command.")]
            private string CreateAlias(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {

                    if (args == null)
                    {
                        this.Print("Expected 2 parameters!");
                        this.Print("Example: createalias >alias name< >command<");
                    }
                    else
                    {
                        if (args.Length >= 2)
                        {
                            if (mobjConsoleReference.CommandManager == null)
                            {
                                this.Print("No command manager is assigned to the console!");
                            }
                            else
                            {
                                string Cmd;
                                Cmd = string.Join(" ", args, 1, args.Length - 1);
                                try
                                {
                                    mobjManager.AddAlias(args[0], Cmd);
                                }
                                catch (ArgumentException arg)
                                {
                                    this.Print(arg.Message);
                                    return null;
                                }

                                this.Print("Alias created. Alias \'" + args[0] + "\' executes >" + Cmd);
                            }
                        }
                        else
                        {
                            this.Print("Alias was not created!");
                            this.Print("Example: createalias >alias name< >command<");
                        }
                    }
                }
                return null;
            }

            [Category("Generic"), Description("Displays the current list of command aliases.")]
            private string ListAlias(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {
                    System.Collections.Specialized.StringCollection Lines = new System.Collections.Specialized.StringCollection();
                    Lines.Add("");
                    Lines.Add("Alias                    Command");
                    Lines.Add("--------------------------------");
                    string[] Items;
                    Items = mobjManager.GetAliasList();
                    string item;
                    foreach (string tempLoopVar_item in Items)
                    {
                        item = tempLoopVar_item;
                        Lines.Add(item + Helpers.Space(Math.Max(25, item.Length) - Math.Min(25, item.Length)) + mobjManager.GetCommandFromAlias(item));
                    }

                    Lines.Add("");
                    if (mobjManager.AliasCount == 1)
                    {
                        Lines.Add(mobjManager.AliasCount.ToString() + " alias.");
                    }
                    else
                    {
                        Lines.Add(mobjManager.AliasCount.ToString() + " aliases.");
                    }

                    // print string
                    this.Print(Lines);

                    // return string that was printed to console
                    string Ret;
                    Lines.Clear();
                    Lines = null;
                    Ret = Helpers.StringCollectionToString(Lines, "\r\n");
                    return Ret;
                }

                return null;
            }

            [Category("Generic"), Description("Specifies the size of the console in percentages it takes up on screen.")]
            private string SetConsoleSize(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {

                    try
                    {
                        if (args == null)
                        {
                            this.Print("MaxConsolePercentage: " + mobjConsoleReference.MaxConsolePercentage.ToString());
                        }
                        else
                        {
                            float NewSize = float.Parse(args[0]);

                            if (NewSize > 100 || NewSize < 5.0F)
                            {
                                this.Print("consolesize must be between 5 and 100");
                            }
                            else
                            {

                                mobjConsoleReference.MaxConsolePercentage = NewSize;
                                if (mobjConsoleReference.CurrentConsolePercentage < mobjConsoleReference.MaxConsolePercentage && mobjConsoleReference.IsVisible)
                                {
                                    mobjConsoleReference.State = Console.ConsoleState.Opening;
                                }
                            }
                        }
                    }
                    catch
                    {
                        //Console.Instance.
                        this.Print("consolesize must be between 5 and 100");
                    }
                }

                return null;
            }

            [Category("Generic"), Description("Specifies the speed at which the console opens and closes.")]
            private string SetConsoleSpeed(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {

                    try
                    {
                        if (args == null)
                        {
                            this.Print("Speed: " + mobjConsoleReference.ConsoleSpeed.ToString());
                        }
                        else
                        {
                            float Value = float.Parse(args[0]);

                            if (Value < 0.0F)
                            {
                                this.Print("Speed must be greater then 0.0");
                            }
                            else
                            {
                                mobjConsoleReference.ConsoleSpeed = Value;
                            }
                        }
                    }
                    catch
                    {
                        //Console.Instance.
                        // mobjExtenderManager.Console.Print("consolesize must be between 5 and 100")
                    }
                }

                return null;
            }

            [Category("Generic"), Description("Clears the console.")]
            private string Clear(string[] args)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {
                    mobjConsoleReference.Entries.Clear();
                    mobjConsoleReference.Entryline = string.Empty;
                }

                return null;
            }
            #endregion

            //Public Sub OnDisconnect() Implements DXTools.IConsoleExtender.OnDisconnect
            //    mobjManager = Nothing
            //    DoRemoveCommands()

            //    mobjCommandList.Clear()
            //    mobjCommandList = Nothing
            //    mblnIsConnected = False
            //    RemoveHandler mobjExtenderManager.Console.CommandManager.BeforeRemoveCommand, AddressOf CommandIsBeingRemoved
            //End Sub

            //Public ReadOnly Property IsConnected() As Boolean Implements DXTools.IConsoleExtender.IsConnected
            //    Get
            //        Return mblnIsConnected
            //    End Get
            //End Property

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                if (mobjManager == null)
                {
                    return;
                }
                DoAddCommand("commands", new xGameConsoleXNA.CommandCallback(this.ListCommands));
                DoAddCommand("echo", new xGameConsoleXNA.CommandCallback(this.Echo));
                DoAddCommand("createalias", new xGameConsoleXNA.CommandCallback(this.CreateAlias));
                DoAddCommand("deletealias", new xGameConsoleXNA.CommandCallback(this.DeleteAlias));
                DoAddCommand("alias", new xGameConsoleXNA.CommandCallback(this.ListAlias));
                DoAddCommand("help", new xGameConsoleXNA.CommandCallback(this.ShowHelp));
                DoAddCommand("cat", new xGameConsoleXNA.CommandCallback(this.ShowCategory));
                DoAddCommand("consolesize", new xGameConsoleXNA.CommandCallback(this.SetConsoleSize));
                DoAddCommand("consolespeed", new xGameConsoleXNA.CommandCallback(this.SetConsoleSpeed));
                DoAddCommand("clear", new xGameConsoleXNA.CommandCallback(this.Clear));
                DoAddCommand("savealias", new xGameConsoleXNA.CommandCallback(SaveAliasList));
                DoAddCommand("loadalias", new xGameConsoleXNA.CommandCallback(LoadAliasList));
            }

            //    Private Sub DoAddCommand(ByVal Name As String, ByVal CB As DXTools.CommandCallback)
            //        If mobjManager Is Nothing Then Exit Sub
            //        If mobjExtenderManager.Console Is Nothing Then Exit Sub
            //        If mobjExtenderManager.Console.CommandManager Is Nothing Then Exit Sub
            //        mobjExtenderManager.Console.CommandManager.AddCommand(Name, CB)
            //        mobjCommandList.Add(Name)
            //    End Sub

            //    Private Sub CommandIsBeingRemoved(ByVal CommandName As String, ByVal e As System.ComponentModel.CancelEventArgs)
            //        ' we can now prevent any of our commands from being removed
            //        e.Cancel = (mobjCommandList.IndexOf(CommandName) <> -1) And (mblnRemovingCommands = False)
            //        If e.Cancel Then
            //            If mblnIsConnected = False Then Exit Sub
            //            If mobjExtenderManager.Console Is Nothing Then Exit Sub
            //            mobjExtenderManager.Console.Print("ConsoleCommands: Some process tried to remove command " & CommandName)
            //        End If
            //    End Sub

            //    Private Sub DoRemoveCommands()
            //        If mobjManager Is Nothing Then Exit Sub
            //        If mobjExtenderManager.Console Is Nothing Then Exit Sub
            //        If mobjExtenderManager.Console.CommandManager Is Nothing Then Exit Sub
            //        mblnRemovingCommands = True
            //        Dim tmp As String
            //        For Each tmp In mobjCommandList
            //            mobjExtenderManager.Console.CommandManager.RemoveCommand(tmp)
            //        Next
            //        mobjCommandList.Clear()
            //        mblnRemovingCommands = False
            //    End Sub

            //#Region " IDisposable Support "
            //    Private disposed As Boolean = False
            //    ' IDisposable
            //    Private Overloads Sub Dispose(ByVal disposing As Boolean)
            //        If Not Me.disposed Then
            //            If disposing Then
            //                DoRemoveCommands()
            //                If mobjCommandList Is Nothing = False Then
            //                    mobjCommandList.Clear()
            //                End If
            //                mobjCommandList = Nothing
            //                If mobjManager Is Nothing = False Then
            //                    If mobjExtenderManager.Console Is Nothing = False Then
            //                        If mobjExtenderManager.Console.CommandManager Is Nothing = False Then
            //                            RemoveHandler mobjExtenderManager.Console.CommandManager.BeforeRemoveCommand, AddressOf CommandIsBeingRemoved
            //                        End If
            //                    End If
            //                End If
            //                mobjManager = Nothing

            //            End If
            //            ' TODO: put code to free unmanaged resources here
            //        End If
            //        Me.disposed = True
            //    End Sub

            //    ' This code added by Visual Basic to correctly implement the disposable pattern.
            //    Public Overloads Sub Dispose() Implements IDisposable.Dispose
            //        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            //        Dispose(True)
            //        GC.SuppressFinalize(Me)
            //    End Sub

            //    Protected Overrides Sub Finalize()
            //        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            //        Dispose(False)
            //        MyBase.Finalize()
            //    End Sub
            //#End Region

            public GenericCommands()
            {
            }

            public GenericCommands(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }
        }
    }

}
