/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using System.ComponentModel;


namespace xGameConsoleXNA
{
	namespace GenericCommands
	{
		
		public class VaribleCommands : BaseCommands
		{
			
			// Implements DXTools.IConsoleExtender, idisposable
			
			private System.Collections.Generic.Dictionary<string, string> mobjVaribles = new System.Collections.Generic.Dictionary<string, string>();
			
			#region " Commands ... "
			[Category("Varibles"), Description("Adds two numeric varibles together. If /n or -n is specified as the last argument no text will be printed out to the console.")]public string AddVar(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length < 2 || args.Length > 3)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A varible with the name " + args[0] + " does not exist!");
					}
					return null;
				}
				if (Helpers.IsNumeric(mobjVaribles[args[0]]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("The varible does not appear to be numeric.");
					}
					return null;
				}
                if (Helpers.IsNumeric(args[1]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A non numeric value was specified. The value was " + args[1]);
					}
					return null;
				}
				// add values together
				float Val1;
				float Val2;
				Val1 = System.Convert.ToSingle(mobjVaribles[args[0]]);
				Val2 = System.Convert.ToSingle(args[1]);
				mobjVaribles[args[0]] = (Val1 + Val2).ToString();
				// return new varible value
				return mobjVaribles[args[0]];
			}
			
			[Category("Varibles"), Description("Appends data to the value of a varible. If /n or -n is specified as the last argument no text will be printed out to the console.")]public string VarAppend(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length < 2 || args.Length > 3)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A varible with the name " + args[0] + " does not exist!");
					}
					return null;
				}
				// append data to varible data
				mobjVaribles[args[0]] += args[1];
				return null;
			}
			
			[Category("Varibles"), Description("Creates a varible. If /n or -n is specified as the last argument no text will be printed out to the console.")]public string CreateVar(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length < 2 || args.Length > 3)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]))
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A varible with the name " + args[0] + " already exists!");
					}
					return null;
				}
				mobjVaribles.Add(args[0], args[1]);
				return null;
			}
			
			[Category("Varibles"), Description("Sets the value of a varible. If /n or -n is specified as the last argument no text will be printed out to the console.")]public string SetVar(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length < 2 || args.Length > 3)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A varible with the name " + args[0] + " does not exist!");
					}
					return null;
				}
				mobjVaribles[args[0]] = args[1];
				return null;
			}
			
			[Category("Varibles"), Description("Gets the value of a varible. If /n or -n is specified as the last argument no text will be printed out to the console.")]public string GetVar(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length < 1 || args.Length > 2)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]) == false)
				{
					if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
					{
						this.Print("A varible with the name " + args[0] + " does not exist!");
					}
					return null;
				}
				
				// Me.Print(args(0) & " equals " & mobjVaribles(args(0)))
				return mobjVaribles[args[0]];
			}
			
			[Category("Varibles"), Description("Deletes a varible.")]public string DeleteVar(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				// process any functions that may be used for arguments
				args = mobjManager.ProcessArguments(args);
				if (args.Length != 1)
				{
					return null;
				}
				
				if (mobjVaribles.ContainsKey(args[0]) == false)
				{
					this.Print("A varible with the name " + args[0] + " does not exist!");
					return null;
				}
				mobjVaribles.Remove(args[0]);
				return null;
			}
			
			[Category("Varibles"), Description("Displays a list of all the created varibles.")]private string ListVaribles(string[] args)
			{
				if (mblnIsConnected == false)
				{
					return null;
				}
				if (mobjManager == null)
				{
					return null;
				}
				
				string[] Varibles;
				Varibles = new string[mobjVaribles.Keys.Count-1 + 1];
				mobjVaribles.Keys.CopyTo(Varibles, 0);
				
				this.PrintBlankLine();
				this.Print("Varible             Value");
				this.Print("------------------------------------------------");
				
				string Tmp = "";
				foreach (string tempLoopVar_Tmp in Varibles)
				{
					Tmp = tempLoopVar_Tmp;
					this.Print(Tmp.PadRight(Math.Max(20, Tmp.Length) - Math.Min(20, Tmp.Length), ' ') + mobjVaribles[Tmp]);
				}
				
				this.PrintBlankLine();
				this.Print(mobjVaribles.Count.ToString() + " varible(s).");
				return null;
			}
			
			
			#endregion
			
			public VaribleCommands()
			{
			}
			
			public VaribleCommands(xGameConsoleXNA.Console Console) : base(Console)
			{
			}
			
			public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
			{
				base.OnConnect(Manager, ExtenderManager);
				DoAddCommand("createvar", new xGameConsoleXNA.CommandCallback( this.CreateVar));
				DoAddCommand("setvar", new xGameConsoleXNA.CommandCallback( this.SetVar));
				DoAddCommand("getvar", new xGameConsoleXNA.CommandCallback( this.GetVar));
				DoAddCommand("deletevar", new xGameConsoleXNA.CommandCallback( this.DeleteVar));
				DoAddCommand("listvars", new xGameConsoleXNA.CommandCallback( this.ListVaribles));
				DoAddCommand("appendvar", new xGameConsoleXNA.CommandCallback( this.VarAppend));
				DoAddCommand("addvar", new xGameConsoleXNA.CommandCallback( this.AddVar));
			}
		}
	}
}
