/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
// Special thanks to Jason Olson from which this code is based on.
// http://geekswithblogs.net/jolson/
// http://geekswithblogs.net/jolson/archive/2004/02/16/2174.aspx
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{
    public delegate string CommandCallback(string[] args);
    //Public Delegate Function FunctionCallback(ByVal args() As String) As Object

    //Public Enum CommandProcessor
    //    Client
    //    Server
    //End Enum

    public class CommandManager : IDisposable
    {


        // Inner Types
        private class Command
        {
            public string Name;
            public CommandCallback Callback;
            public string[] Arguments;
        }

        public const char StartFunction = '[';
        public const char EndFunction = ']';

        public delegate void BeforeCommandExecutedEventHandler(object Sender, ReturnValueEventArgs e);
        private BeforeCommandExecutedEventHandler BeforeCommandExecutedEvent;

        public event BeforeCommandExecutedEventHandler BeforeCommandExecuted
        {
            add
            {
                BeforeCommandExecutedEvent = (BeforeCommandExecutedEventHandler)System.Delegate.Combine(BeforeCommandExecutedEvent, value);
            }
            remove
            {
                BeforeCommandExecutedEvent = (BeforeCommandExecutedEventHandler)System.Delegate.Remove(BeforeCommandExecutedEvent, value);
            }
        }

        public delegate void AfterCommandExecutedEventHandler(object Sender, ReturnValueEventArgs e);
        private AfterCommandExecutedEventHandler AfterCommandExecutedEvent;

        public event AfterCommandExecutedEventHandler AfterCommandExecuted
        {
            add
            {
                AfterCommandExecutedEvent = (AfterCommandExecutedEventHandler)System.Delegate.Combine(AfterCommandExecutedEvent, value);
            }
            remove
            {
                AfterCommandExecutedEvent = (AfterCommandExecutedEventHandler)System.Delegate.Remove(AfterCommandExecutedEvent, value);
            }
        }

        public delegate void BeforeRemoveCommandEventHandler(string CommandName, System.ComponentModel.CancelEventArgs e);
        private BeforeRemoveCommandEventHandler BeforeRemoveCommandEvent;

        public event BeforeRemoveCommandEventHandler BeforeRemoveCommand
        {
            add
            {
                BeforeRemoveCommandEvent = (BeforeRemoveCommandEventHandler)System.Delegate.Combine(BeforeRemoveCommandEvent, value);
            }
            remove
            {
                BeforeRemoveCommandEvent = (BeforeRemoveCommandEventHandler)System.Delegate.Remove(BeforeRemoveCommandEvent, value);
            }
        }

        public delegate void BeforeRemoveAliasEventHandler(string AliasName, System.ComponentModel.CancelEventArgs e);
        private BeforeRemoveAliasEventHandler BeforeRemoveAliasEvent;

        public event BeforeRemoveAliasEventHandler BeforeRemoveAlias
        {
            add
            {
                BeforeRemoveAliasEvent = (BeforeRemoveAliasEventHandler)System.Delegate.Combine(BeforeRemoveAliasEvent, value);
            }
            remove
            {
                BeforeRemoveAliasEvent = (BeforeRemoveAliasEventHandler)System.Delegate.Remove(BeforeRemoveAliasEvent, value);
            }
        }

        public delegate void BeforeCaseSensitivityChangeEventHandler(object Sender, System.ComponentModel.CancelEventArgs e);
        private BeforeCaseSensitivityChangeEventHandler BeforeIgnoreCaseChangeEvent;

        public event BeforeCaseSensitivityChangeEventHandler BeforeCaseSensitivityChange
        {
            add
            {
                BeforeIgnoreCaseChangeEvent = (BeforeCaseSensitivityChangeEventHandler)System.Delegate.Combine(BeforeIgnoreCaseChangeEvent, value);
            }
            remove
            {
                BeforeIgnoreCaseChangeEvent = (BeforeCaseSensitivityChangeEventHandler)System.Delegate.Remove(BeforeIgnoreCaseChangeEvent, value);
            }
        }

        public delegate void AfterCaseSensitivityChangedEventHandler(object Sender, EventArgs e);
        private AfterCaseSensitivityChangedEventHandler AfterIgnoreCaseChangedEvent;

        public event AfterCaseSensitivityChangedEventHandler AfterCaseSensitivityChanged
        {
            add
            {
                AfterIgnoreCaseChangedEvent = (AfterCaseSensitivityChangedEventHandler)System.Delegate.Combine(AfterIgnoreCaseChangedEvent, value);
            }
            remove
            {
                AfterIgnoreCaseChangedEvent = (AfterCaseSensitivityChangedEventHandler)System.Delegate.Remove(AfterIgnoreCaseChangedEvent, value);
            }
        }


        // delegates
        public delegate string ReturnValueCallback(string Command, int Index);

        // Attributes/Properties
        private CommandBuffer mobjCommandBuffer = new CommandBuffer();
        private Dictionary<string, CommandCallback> mobjCommands = new Dictionary<string, CommandCallback>();
        private Dictionary<string, string> mobjAlias = new Dictionary<string, string>();
        private Console mobjConsole;
        // Private _HasBeenInit As Boolean = False
        protected bool ignoreCase = true;
        protected internal bool mblnAutoProcessArgs = true;


        public bool AutoProcessArguments
        {
            get { return mblnAutoProcessArgs; }
            set { mblnAutoProcessArgs = value; }
        }

        /// <summary>
        /// Controls weather commads executed via the Execute method are check for case sensitivity.
        /// </summary>
        /// <value>Returns True if case sensitivity is turned on, False if it is not.</value>
        /// <remarks></remarks>
        public bool IgnoreCase
        {
            get { return ignoreCase; }
            set
            {
                if (value == ignoreCase) return;

                try
                {
                    System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);
                    if (BeforeIgnoreCaseChangeEvent != null)
                        BeforeIgnoreCaseChangeEvent(this, e);
                    if (e.Cancel) return;
                }
                catch
                {
                }

                ignoreCase = value;

                try
                {
                    if (AfterIgnoreCaseChangedEvent != null)
                        AfterIgnoreCaseChangedEvent(this, new EventArgs());
                }
                catch
                {
                }
            }
        }

        public string GetCommandFromAlias(string AliasName)
        {
            if (AliasExists(AliasName, true) == false)
            {
                return null;
            }
            return mobjAlias[AliasName];
        }

        public bool AliasExists(string AliasName, bool CaseSensitive)
        {
            return (DoAliasExists(AliasName, true) != null);
            //If mblnCaseSensitivity Or CaseSensitive Then
            //    For Each C As String In _commands.Keys
            //        If C.ToLower = AliasName.ToLower Then
            //            Return True
            //        End If
            //    Next
            //    Return False
            //Else
            //    Return _alias.ContainsKey(AliasName)
            //End If
        }

        public bool CommandExists(string CommandName)
        {
            return CommandExists(CommandName, false);
        }

        public bool CommandExists(string CommandName, bool searchAliases)
        {
            if ((DoCommandExists(CommandName, true) != null) || (DoAliasExists(CommandName, this.IgnoreCase) == null)) return false;
            return true;
        }

        public bool CommandExists(string CommandName, bool searchAliases, bool ignoreCase)
        {
            if ((DoCommandExists(CommandName, true) != null) && (DoAliasExists(CommandName, ignoreCase) == null)) return false;
            return true;
        }

        protected internal string DoAliasExists(string AliasName, bool ignoreCasing)
        {
            if (ignoreCasing)
            {
                foreach (string C in mobjAlias.Keys)
                {
                    if (string.Compare(C, AliasName, ignoreCasing) == 0) return C;
                }
                return null;
            }
            else
            {
                if (mobjAlias.ContainsKey(AliasName))
                {
                    return AliasName;
                }
                else
                {
                    return null;
                }
            }
        }

        protected internal CommandCallback DoCommandExists(string CommandName, bool ignoreCase)
        {
            if (ignoreCase)
            {
                foreach (string C in mobjCommands.Keys)
                {
                    if (string.Compare(C, CommandName, ignoreCase) == 0) return mobjCommands[C];

                }
                return null;
            }
            else
            {
                if (mobjCommands.ContainsKey(CommandName)) return mobjCommands[CommandName];
                return null;
            }
        }

        /// <summary>
        /// Gets an array of category names for the different commands registered with the console.
        /// </summary>
        /// <returns>Returns an array of category names.</returns>
        public string[] GetCategoryList()
        {
            return GetCategoryList(false);
        }

        /// <summary>
        /// Gets an array of category names for the different commands registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also include commands that have an VisibleToUsers attribute this is set to false.</param>
        /// <returns>Returns an array of category names.</returns>
        public string[] GetCategoryList(bool includeHidden)
        {
            // if no commands loaded return null
            if (mobjCommands.Count == 0) return null;

            System.Collections.Specialized.StringCollection items = new System.Collections.Specialized.StringCollection();

            // loop thru all commands
            foreach (string key in mobjCommands.Keys)
            {
                // get command
                CommandCallback cx;
                cx = mobjCommands[key];

                // check if category attribute was specified
                System.ComponentModel.CategoryAttribute cat;
                cat = Helpers.GetAttribute<System.ComponentModel.CategoryAttribute>(cx.Method);
                if (cat != null)
                {
                    // check if visibletousers attribute was specified
                    VisibleToUsers vis;
                    vis = Helpers.GetAttribute<VisibleToUsers>(cx.Method);
                    if (vis != null)
                    {
                        // check to add hidden
                        if (vis.Visible == false && includeHidden)
                        {
                            if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                        }
                        else
                        {
                            if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                        }
                    }
                    else
                    {
                        if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                    }
                }
            }

            // copy commands to array and return
            string[] cmds;
            cmds = new string[items.Count];
            items.CopyTo(cmds, 0);
            items.Clear();
            items = null;
            return cmds;
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList()
        {
            return GetCommandList(false);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an VisibleToUsers attribute this is set to false.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(bool includeHidden)
        {
            return GetCommandList(string.Empty, includeHidden);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an VisibleToUsers attribute this is set to false.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(string cat)
        {
            return GetCommandList(cat, true, false);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an VisibleToUsers attribute this is set to false.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(string cat, bool includeHidden)
        {
            return GetCommandList(cat, true, includeHidden);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an VisibleToUsers attribute this is set to false.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(string category, bool ignoreCase, bool includeHidden)
        {
            // if no commands loaded return null
            if (mobjCommands.Count == 0) return null;

            // loop thru all commands
            System.Collections.Specialized.StringCollection items = new System.Collections.Specialized.StringCollection();
            foreach (string key in mobjCommands.Keys)
            {
                // get command
                CommandCallback cx;
                cx = mobjCommands[key];

                // try to get VisibleToUsers attribute
                VisibleToUsers vis;
                vis = Helpers.GetAttribute<VisibleToUsers>(cx.Method);

                // try to get Category attribute
                System.ComponentModel.CategoryAttribute cat;
                cat = Helpers.GetAttribute<System.ComponentModel.CategoryAttribute>(cx.Method);

                // check if command has been flagged invisible to console users and if we want to include the hidden commands
                if (vis != null && vis.Visible == false && includeHidden)
                {
                    // check if a category has been specified for command
                    if (cat != null && category != string.Empty)
                    {
                        // if category matches then add it
                        if (string.Compare(cat.Category, category, ignoreCase) == 0)
                        {
                            items.Add(key);
                        }
                    }
                    else // no category specified so add anyway
                    {
                        items.Add(key);
                    }
                }
                else // visibletousers attribute is not present or is set to true
                {
                    // check if a category has been specified for command
                    if (cat != null && category != string.Empty)
                    {
                        // if category matches then add it
                        if (string.Compare(cat.Category, category, ignoreCase) == 0)
                        {
                            items.Add(key);
                        }
                    }
                    else  // no category specified so add anyway
                    {
                        items.Add(key);
                    }
                }
            }

            // copy commands to array and return
            string[] cmds;
            cmds = new string[items.Count];
            items.CopyTo(cmds, 0);
            items.Clear();
            items = null;
            return cmds;
        }

        /// <summary>
        /// Gets an array of command aliases.
        /// </summary>
        /// <returns>Returns an array of command aliases.</returns>
        public string[] GetAliasList()
        {
            if (mobjAlias.Count == 0)
            {
                return null;
            }
            string[] cmds;
            cmds = new string[mobjAlias.Count - 1 + 1];
            mobjAlias.Keys.CopyTo(cmds, 0);
            return cmds;
        }

        /// <summary>
        /// Gets the number of command aliases.
        /// </summary>
        /// <returns>Returns the number of aliases in the aliases list.</returns>
        public int AliasCount
        {
            get { return mobjAlias.Count; }
        }

        /// <summary>
        /// Gets the number of commands registered with the console.
        /// </summary>
        /// <returns>Returns the number of commands registered with the console.</returns>
        public int CommandCount
        {
            get { return mobjCommands.Count; }
        }

        public CommandCallback GetCommand(string CommandName, bool CheckAliases)
        {
            CommandCallback Cmd;

            Cmd = DoCommandExists(CommandName, true);
            if (Cmd != null)
            {
                return Cmd;
            }
            if (CheckAliases)
            {
                string cName;
                cName = DoAliasExists(CommandName, true);
                if (cName == null)
                {
                    return null;
                }
                Cmd = DoCommandExists(mobjAlias[cName], true);
                if (Cmd != null)
                {
                    return Cmd;
                }

                //If _alias.ContainsKey(Cmd) Then
                //    Cmd = _alias(Cmd)
                //Else
                //    ' mobjManager.Console.Print("No command or command alias with that name exists!")
                //    'Exit Function
                //    Return Nothing
                //End If
            }
            else
            {
                return null;
            }

            return null; //_commands(Cmd)
        }

        public CommandBuffer Buffer
        {
            get { return mobjCommandBuffer; }
        }

        // no property accessor

        // Singleton Implementation
        // Public Shared ReadOnly Instance As CommandManager = New CommandManager()
        public CommandManager()
        {

        }

        public CommandManager(Console Console)
        {
            mobjConsole = Console;
        }

        public CommandManager(bool ignoreCasing)
        {
            ignoreCase = ignoreCasing;
        }

        public CommandManager(Console Console, bool ignoreCasing)
        {
            mobjConsole = Console;
            ignoreCase = ignoreCasing;
        }

        // Operations/Methods
        //Public Sub Initialize()
        //     _HasBeenInit = True
        //End Sub

        public void AddAlias(string AliasName, string Command)
        {

            // check if valid alias name
            AliasName = AliasName.Trim();
            if (IsValidCommandName(AliasName) == false)
            {
                throw (new ArgumentException(Resources.AliasNameIsNotValid.Replace(Resources.RKEY, AliasName), "AliasName"));
            }

            // check if alias already exists
            if (AliasExists(AliasName, true))
            {
                throw (new ArgumentException(Resources.AliasNameAlreadyExists.Replace(Resources.RKEY, AliasName), "AliasName"));
            }
            else
            {
                // add alias
                mobjAlias.Add(AliasName, Command);
            }
        }

        //Public Sub AddCommand(ByVal name As String, ByVal callback As CommandCallback)
        //    AddCommand(name, callback, CommandProcessor.Client)
        //End Sub

        public static bool IsValidCommandName(string CommandName)
        {
            string lowStr = "abcdefghijklmnopqrstuuvwxyz";
            string uppStr = lowStr.ToUpper();
            char[] lowChr = lowStr.ToCharArray();
            char[] uppChr = uppStr.ToCharArray();
            foreach (char c in CommandName)
            {
                if ((c.ToString().IndexOfAny(lowChr) != -1) &&
                    (c.ToString().IndexOfAny(uppChr) != -1) &&
                    (c != '_')) return false;
            }
            return true;

            //if (CommandName == null || CommandName == string.Empty)
            //{
            //    return false;
            //}
            //if (CommandName.IndexOfAny(@"!@#$%^&*()[]{}\\|\'\"";:<>,./?`~+=- ".ToCharArray()) != - 1)
            //{
            //    return false; // check for bad chars
            //}
            //System.Text.RegularExpressions.Regex reg = new System.Text.RegularExpressions.Regex("[0-9]*");
            //if (reg.Match(CommandName).Success)
            //{
            //    return false; // if availible name starts with a number
            //}

            ////if (StringType.StrLike(CommandName, "[0-9]*", CompareMethod.Binary))
            ////{
            ////    return false; // check if availible name starts with a number
            ////}
            //if (StringType.StrLike(CommandName,("*[" + '\0' + "-" + '\u001F' + "]*"), CompareMethod.Binary))
            //{
            //    return false;
            //}
            //if (StringType.StrLike(CommandName,("*[" + '\u007B' + "-" + Strings.Chr(255) + "]*"), CompareMethod.Binary))
            //{
            //    return false;
            //}

            //return true;

            ////==========
            //// I keep forgeting about the like operator! this was the old validation code
            ////==========

            //// check first character to ensure that it is a valid starting character
            ////switch( CommandName[0])
            ////{
            ////    case 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'u', 'v', 'w', 'x', 'y', 'z':
            ////        break;
            ////    case "A"c To "Z"c
            ////        break;

            ////    case '_'
            ////          break;

            ////    default:
            ////        return false;
            ////}

            ////' check subsequent characters to ensure that it is a valid command name
            ////For Each c As Char In CommandName
            ////    Select Case c
            ////        Case "a"c To "z"c
            ////        Case "A"c To "Z"c
            ////        Case "_"c
            ////        Case "0"c To "9"c

            ////        Case Else
            ////            Return False
            ////    End Select
            ////Next

            ////Return True
        }

        public void AddCommand(string CommandName, CommandCallback callback) //, ByVal processor As CommandProcessor)
        {
            CommandName = CommandName.Trim();
            if (IsValidCommandName(CommandName) == false)
            {
                throw (new ArgumentException("Command " + CommandName + " is not a valid command name.", "CommandName"));
            }

            if (callback == null)
            {
                throw (new ArgumentNullException("Callback"));
                //  If Not _console Is Nothing Then _console.PrintFormat("command {0} must have a callback", name)
            }
            else
            {
                if (CommandExists(CommandName))
                {
                    throw (new ArgumentException("Command " + CommandName + " already exists.", "CommandName"));
                    // If Not _console Is Nothing Then _console.PrintFormat("command {0} already exists", name)
                }
                else
                {
                    //  _commands.Add(CommandName, New CommandDefinition(CommandName, callback)) ', processor))
                    mobjCommands.Add(CommandName, callback);
                }
            }
        }

        private bool DoRemoveCommand(string CommandName)
        {
            System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);
            try
            {
                if (BeforeRemoveCommandEvent != null)
                    BeforeRemoveCommandEvent(CommandName, e);
            }
            catch
            {
            }
            if (e.Cancel)
            {
                return false;
            }
            mobjCommands[CommandName] = null;
            mobjCommands.Remove(CommandName);
            return true;
        }

        public void RemoveCommand(string CommandName)
        {
            if (this.ignoreCase)
            {
                foreach (string C in mobjCommands.Keys)
                {
                    if (string.Compare(C, CommandName, this.ignoreCase) == 0)
                    {
                        DoRemoveCommand(C);
                        return; //_commands(C)
                    }
                }
                throw (new ArgumentException(Resources.SpecifiedCommandNotExist, "CommandName"));
                //return; //Nothing
            }
            else
            {
                DoRemoveCommand(CommandName);
                return; //_commands(CommandName)
            }

            //If CommandExists(CommandName) Then


            //Else
            //    Throw New ArgumentException("Command " & CommandName & " does not exist.", "CommandName")
            //    ' If Not _console Is Nothing Then _console.PrintFormat("Command {0} does not exist", CommandName)
            //End If
        }

        /// <summary>
        /// Will return False if event BeforeRemoveAlias canceled, otherwise will return True.
        /// </summary>
        /// <param name="AliasName">The name if the name of the command alias.</param>
        /// <returns></returns>
        /// <remarks></remarks>
        private bool DoRemoveAlias(string AliasName)
        {
            System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);
            try
            {
                if (BeforeRemoveAliasEvent != null)
                    BeforeRemoveAliasEvent(AliasName, e);
            }
            catch
            {
            }
            if (e.Cancel)
            {
                return false;
            }
            mobjAlias.Remove(AliasName);
            return true;
        }

        public void RemoveAlias(string AliasName)
        {
            if (this.ignoreCase)
            {
                foreach (string C in mobjAlias.Keys)
                {
                    if (string.Compare(C, AliasName, this.ignoreCase) == 0)
                    {
                        DoRemoveAlias(C);
                        return; //_commands(C)
                    }
                }
                throw (new ArgumentException("Alias " + AliasName + " does not exist.", "AliasName"));
                //return; //Nothing
            }
            else
            {
                DoRemoveAlias(AliasName);
                return; //_commands(CommandName)
            }
        }


        public string Execute(System.Collections.Specialized.StringCollection Commands, ReturnValueCallback ReturnValueCB)
        {
            if (Commands == null)
            {
                return null;
            }
            for (int idx = 0; idx <= Commands.Count - 1; idx++)
            {
                string Val;
                Val = DoExecuteString(Commands[idx], true);
                try
                {
                    if (ReturnValueCB != null)
                    {
                        ReturnValueCB(Commands[idx], idx);
                    }
                }
                catch
                {
                }
            }
            return null;
        }

        public string Execute(string Command, ReturnValueCallback ReturnValueCB)
        {
            return this.Execute(new string[] { Command }, ReturnValueCB);
        }

        public string Execute(string[] Commands, ReturnValueCallback ReturnValueCB)
        {
            if (Commands == null)
            {
                return null;
            }
            for (int idx = 0; idx <= Commands.Length - 1; idx++)
            {
                string Val;
                Val = DoExecuteString(Commands[idx], true);
                try
                {
                    if (ReturnValueCB != null)
                    {
                        ReturnValueCB(Commands[idx], idx);
                    }
                }
                catch
                {
                }
            }
            return null;
        }

        public string Execute(string Command)
        {
            return DoExecuteString(Command, true);
        }

        public string Execute(int Count)
        {
            // If Count = 0 Then Return Nothing
            // execute all qued commands
            //If Count < 0 Then
            //    While _commandBuffer.Lines.Count > 0
            //        ExecuteString(_commandBuffer.Lines.Dequeue().ToString())
            //    End While

            //Else   ' execute range of commands
            int idx;
            if (Count < 1)
            {
                Count = mobjCommandBuffer.Lines.Count;
            }
            for (idx = 0; idx <= Count - 1; idx++)
            {
                // If _commandBuffer.Lines.Count = 0 Then Exit For
                if (idx == (Count - 1))
                {
                    return DoExecuteString(mobjCommandBuffer.Lines.Dequeue().ToString(), true);
                }
                else
                {
                    DoExecuteString(mobjCommandBuffer.Lines.Dequeue().ToString(), true);
                }
            }
            // End If

            return null;
        }

        public string ExecuteString(string command)
        {
            return DoExecuteString(command, true);
        }

        private bool RaiseBeforeCommandExecuted(string Command, string[] Args, string ReturnValue, bool Cancel)
        {
            ReturnValueEventArgs e = new ReturnValueEventArgs(Command, Args, ReturnValue, Cancel);
            try
            {
                if (BeforeCommandExecutedEvent != null)
                    BeforeCommandExecutedEvent(this, e);
            }
            catch
            {
            }
            return e.Cancel;
        }

        private bool RaiseAfterCommandExecuted(string Command, string[] Args, string ReturnValue, bool Cancel)
        {
            ReturnValueEventArgs e = new ReturnValueEventArgs(Command, Args, ReturnValue, Cancel);
            try
            {
                if (AfterCommandExecutedEvent != null)
                    AfterCommandExecutedEvent(this, e);
            }
            catch
            {
            }
            return e.Cancel;
        }

        private string DoExecuteString(string CommandString, bool RaiseExecutedEvent)
        {
            Command command;
            string Ret = null;

            // retreve the command from the text
            command = ParseCommandText(CommandString);

            // check to make sure call back exists
            if (command != null && command.Callback != null)
            {
                // check to raise event
                if (RaiseExecutedEvent)
                {
                    // raise event and return nothig if canceled
                    if (RaiseBeforeCommandExecuted(command.Name, command.Arguments, null, false))
                    {
                        return null;
                    }
                }

                // call method
                try
                {
                    if (mblnAutoProcessArgs)
                    {
                        command.Arguments = this.ProcessArguments(command.Arguments);
                    }
                    Ret = command.Callback(command.Arguments);
                }
                catch
                {
                }

                // check to raise event
                if (RaiseExecutedEvent)
                {
                    // raise event and return nothig if canceled
                    if (RaiseAfterCommandExecuted(command.Name, command.Arguments, Ret, false))
                    {
                        return null;
                    }
                }
            }

            return Ret;
        }

        public string[] ProcessArguments(System.Collections.Specialized.StringCollection Args)
        {
            System.Collections.Specialized.StringCollection Params = new System.Collections.Specialized.StringCollection();
            // Dim OldParams As New Specialized.StringCollection

            if (Args == null)
            {
                return null;
            }
            // OldParams.AddRange(Args)

            // process args
            while (Args.Count > 0)
            {
                switch (Args[0][0])
                {
                    case StartFunction:

                        Args.RemoveAt(0);
                        try
                        {
                            string[] Value;
                            Value = ProcessArguments(Args);

                            // Dim Value As String
                            // Value = Me.ExecuteString(String.Join(" ", Command))

                            @Params.AddRange(Value);

                        }
                        catch (ArgumentNullException)
                        {
                            // the returned array was nothing
                        }
                        break;

                    case EndFunction:

                        Args.RemoveAt(0);
                        string pValue;
                        pValue = this.DoExecuteString(Helpers.StringCollectionToString(@Params, " "), true);

                        return new string[] { pValue };

                    default:

                        Params.Add(Args[0]);
                        Args.RemoveAt(0);
                        break;
                }
            }

            // return new args
            return Helpers.StringCollectionToArray(Params);
        }

        /// <summary>
        /// Will process the argument array for any function definitions, converting them to literal values.
        /// </summary>
        /// <param name="Args">A list of arguments to process.</param>
        /// <returns>Will return the converted function call arguments into there literal values.</returns>
        /// <remarks>
        /// Will turn "[ GetSomeValue ]" into a literal value of "123".
        /// <example>
        /// For example the arguments of the echo command ...
        ///
        /// echo [ GetSomeValue ]
        ///
        /// ... become ...
        ///
        /// echo 123
        /// </example>
        /// </remarks>
        public string[] ProcessArguments(string[] Args)
        {
            if (Args == null)
            {
                return null;
            }
            System.Collections.Specialized.StringCollection SC = new System.Collections.Specialized.StringCollection();
            SC.AddRange(Args);
            string[] NewArgs;
            NewArgs = ProcessArguments(SC);
            SC = null;
            return NewArgs;
        }

        private Command ParseCommandText(string commandText)
        {
            Command command = new Command();

            commandText = commandText.Trim();
            string name = "";
            string[] arguments;
            if (commandText == "")
            {
                command.Callback = null;
                return null;
            }

            int index = commandText.IndexOf(" ");
            if (index > 0)
            {
                // we have arguments to parse out
                name = commandText.Substring(0, index);
                string parameters = commandText.Remove(0, index + 1);
                arguments = parameters.Split(' ');
            }
            else
            {
                // there are no arguments so all we have is the command name
                name = commandText;
                arguments = null;
            }

            command.Name = name;
            command.Arguments = arguments;
            command.Callback = DoCommandExists(name, true);
            if (command.Callback == null)
            {
                string cName;
                cName = DoAliasExists(name, true);
                if (cName != null)
                {
                    command = this.ParseCommandText(mobjAlias[cName]);                                       
                }
            }

            return command;
        }

        // Public Sub Initialize(ByVal targetControl As System.Windows.Forms.Control)
        //  _targetControl = targetControl

        // create subsystems
        //  _renderer = New Renderer(targetControl)

        // initialize subsystems
        //  CommandManager.Instance.Initialize()
        // End Sub

        //Public Sub Frame()
        //    CommandManager.Instance.Execute()

        //    Render()
        //End Sub

        //Public ReadOnly Property HasBeenInitialized() As Boolean
        //    Get
        //        Return _HasBeenInit
        //    End Get
        //End Property

        #region " IDisposable Support "
        private bool disposedValue = false; // To detect redundant calls

        // IDisposable
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    // free unmanaged resources when explicitly called
                    mobjCommandBuffer.Dispose();
                    mobjCommandBuffer = null;
                    mobjCommands.Clear();
                    mobjCommands = null;
                    mobjAlias.Clear();
                    mobjAlias = null;
                    mobjConsole = null;
                }

                // free shared unmanaged resources
            }
            this.disposedValue = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion

    }
}
