/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
// Special thanks to Jason Olson from which this code is based on.
// http://geekswithblogs.net/jolson/
// http://geekswithblogs.net/jolson/archive/2004/02/16/2174.aspx
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using Microsoft.Xna.Framework;

namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides a delegate for console rendering.
    /// </summary>
    /// <param name="gameTime">Time passed since the last call to Microsoft.Xna.Framework.Game.Update.</param>
    /// <example>A simplistic example of how to use the Console.RenderConsoleMethod property.
    /// <code>
    /// xGameConsoleXNA.Console console;
    /// console.RenderConsoleMethod = this.Render();
    /// 
    /// void Render(GameTime gameTime) {
    ///     // Place render code here
    /// }
    /// </code>
    /// </example>
    public delegate void RenderConsole(GameTime gameTime);

    public class Console : IDisposable
    {


        /// <summary>
        /// Defines different opening and closing states for the console.
        /// </summary>
        public enum ConsoleState
        {
            /// <summary>
            /// The console is closed.
            /// </summary>
            Closed,
            /// <summary>
            /// THe console is closing.
            /// </summary>
            Closing,
            /// <summary>
            /// The console is opening.
            /// </summary>
            Opening,
            /// <summary>
            /// THe console is open.
            /// </summary>
            Open
        }


        public delegate void BeforeProcessEntryLineEventHandler(object Sender, ProcessEntryLineEventArgs e);
        private BeforeProcessEntryLineEventHandler BeforeProcessEntryLineEvent;

        public event BeforeProcessEntryLineEventHandler BeforeProcessEntryLine
        {
            add
            {
                BeforeProcessEntryLineEvent = (BeforeProcessEntryLineEventHandler)System.Delegate.Combine(BeforeProcessEntryLineEvent, value);
            }
            remove
            {
                BeforeProcessEntryLineEvent = (BeforeProcessEntryLineEventHandler)System.Delegate.Remove(BeforeProcessEntryLineEvent, value);
            }
        }

        public delegate void AfterProcessEntryLineEventHandler(object Sender, TextEventArgs e);
        private AfterProcessEntryLineEventHandler AfterProcessEntryLineEvent;

        public event AfterProcessEntryLineEventHandler AfterProcessEntryLine
        {
            add
            {
                AfterProcessEntryLineEvent = (AfterProcessEntryLineEventHandler)System.Delegate.Combine(AfterProcessEntryLineEvent, value);
            }
            remove
            {
                AfterProcessEntryLineEvent = (AfterProcessEntryLineEventHandler)System.Delegate.Remove(AfterProcessEntryLineEvent, value);
            }
        }

        public delegate void BeforePrintEventHandler(object Sender, PrintEventArgs e);
        private BeforePrintEventHandler BeforePrintEvent;

        public event BeforePrintEventHandler BeforePrint
        {
            add
            {
                BeforePrintEvent = (BeforePrintEventHandler)System.Delegate.Combine(BeforePrintEvent, value);
            }
            remove
            {
                BeforePrintEvent = (BeforePrintEventHandler)System.Delegate.Remove(BeforePrintEvent, value);
            }
        }

        public delegate void AfterPrintEventHandler(object Sender, TextEventArgs e);
        private AfterPrintEventHandler AfterPrintEvent;

        public event AfterPrintEventHandler AfterPrint
        {
            add
            {
                AfterPrintEvent = (AfterPrintEventHandler)System.Delegate.Combine(AfterPrintEvent, value);
            }
            remove
            {
                AfterPrintEvent = (AfterPrintEventHandler)System.Delegate.Remove(AfterPrintEvent, value);
            }
        }



        private int mintAskingQuestion = 0;
        private string mstrLastResponceToQuestion = string.Empty;
        private CommandCallback mobjUserResponce;
        private RenderConsole mobjRenderConsoleMethod;
        private CommandManager mobjCommandManager;
        private string mstrCancelString = "//cancel//";
        protected internal bool mblnAnimateConosle = true;
        private float msngCurrentConsolePercentage = 0.0F;
        private float msngMaxConsolePercentage = 100.0F;
        private float msngConsoleSpeed = 1f;
        private System.Collections.Specialized.StringCollection mobjEntries = new System.Collections.Specialized.StringCollection();
        private int mintMaxEntries = 50;
        private System.Text.StringBuilder mobjEntryLine = new System.Text.StringBuilder(">");
        private ConsoleState mintConsoleState = ConsoleState.Closed;
 
         /// <summary>
         /// Gets/Sets weather or not the console will be animated when opening and closing.
         /// </summary>
        public bool AnimateConosle
        {
            get { return mblnAnimateConosle; }
            set {  mblnAnimateConosle = value; }
        }

        /// <summary>
        /// Gets/Sets the method that will be called by the Console.Render method.
        /// </summary>
        public RenderConsole RenderConsoleMethod
        {
            get { return mobjRenderConsoleMethod;  }
            set { mobjRenderConsoleMethod = value; }
        }


        public void AskQuestion(string Question, CommandCallback Responce)
        {
            this.Print(Question);
            AskQuestion(Responce);
        }

        public void AskQuestion(CommandCallback Responce)
        {
            if (Responce == null)
            {
                return;
            }
            mintAskingQuestion++;
            mobjUserResponce = null;
            mobjUserResponce = Responce;
        }

        private void DoCancelQuestion(bool DoCancel)
        {
            if (mintAskingQuestion == 0)
            {
                return;
            }
            mintAskingQuestion = Math.Max(mintAskingQuestion - 1, 0);
            mobjEntryLine.Remove(1, mobjEntryLine.Length - 1);

            if (DoCancel)
            {
                try
                {
                    mobjUserResponce(new string[] { mstrCancelString }); // report back with cancel responce
                }
                catch
                {
                }
            }
            if (mintAskingQuestion == 0)
            {
                mobjUserResponce = null;
            }
        }

        public void CancelQuestion()
        {
            DoCancelQuestion(true);
        }

        /// <summary>
        /// Returns true if the console is waiting for a responce from the user.
        /// </summary>
        public bool IsAskingQuestion
        {
            get { return mintAskingQuestion != 0; }
        }

        public string Entryline
        {
            get { return mobjEntryLine.ToString(); }
            set
            {
                mobjEntryLine.Remove(1, mobjEntryLine.Length - 1);
                mobjEntryLine.Append(value);
            }
        }

        /// <summary>
        /// Gets a collectin of entries entered into the console.
        /// </summary>
        public System.Collections.Specialized.StringCollection Entries
        {
            get { return mobjEntries; }
        }

        public int MaxEntries
        {
            get
            {
                return mintMaxEntries;
            }
            set
            {
                mintMaxEntries = value;
                if (mintMaxEntries < 1)
                {
                    mintMaxEntries = 1;
                }
            }
        }

        public float MaxConsolePercentage
        {
            get
            {
                return msngMaxConsolePercentage;
            }
            set
            {
                msngMaxConsolePercentage = value;
                if (msngMaxConsolePercentage < 1)
                {
                    msngMaxConsolePercentage = 1;
                }
                if (msngMaxConsolePercentage > 100)
                {
                    msngMaxConsolePercentage = 100;
                }
                if (msngCurrentConsolePercentage > msngMaxConsolePercentage)
                {
                    msngCurrentConsolePercentage = msngMaxConsolePercentage;
                }

            }
        }

        public bool IsVisible
        {
            get
            {
                return mintConsoleState != ConsoleState.Closed;
            }
        }

        public float CurrentConsolePercentage
        {
            get
            {
                return msngCurrentConsolePercentage;
            }
            set
            {
                msngCurrentConsolePercentage = value;
                if (msngCurrentConsolePercentage < 0)
                {
                    msngCurrentConsolePercentage = 0;
                }
                if (msngCurrentConsolePercentage > msngMaxConsolePercentage)
                {
                    msngCurrentConsolePercentage = msngMaxConsolePercentage;
                }
            }
        }

   
        // Singleton Implementation
        //  Public Shared ReadOnly Instance As Console = New Console()

        public Console()
        {   
        }

        public Console(CommandManager CommandManager)
        {
            mobjCommandManager = CommandManager;
        }

        public Console(RenderConsole RenderMethod)
        {
            this.RenderConsoleMethod = RenderMethod;
        }

        public Console(CommandManager CommandManager, RenderConsole RenderMethod)
        {
            this.RenderConsoleMethod = RenderMethod;
            mobjCommandManager = CommandManager;
        }

        //Public Sub Initialize()
        //    '  _windowsFont = New System.Drawing.Font("Arial", 8.0F)
        //    '  _font = New Font(BattleEngine.Instance.Renderer.Device, _windowsFont)

        //    ' add our needed commands
        //     _HasBeenInit = True
        //End Sub

        public void Print(string[] text)
        {
            string Tmp;
            foreach (string tempLoopVar_Tmp in text)
            {
                Tmp = tempLoopVar_Tmp;
                this.Print(Tmp);
            }
        }

        public void Print(System.Collections.Specialized.StringCollection text)
        {
            if (text == null)
            {
                return;
            }
            string Tmp;
            foreach (string tempLoopVar_Tmp in text)
            {
                Tmp = tempLoopVar_Tmp;
                this.Print(Tmp);
            }
        }

        public void Print(string text)
        {
            // check if printing multiple lines
            int pos = text.IndexOf("\n");
            if (pos == -1)
            {
                this.DoPrint(text);
                text = string.Empty;
            }
            else
            {
                while (pos != -1)
                {
                    this.DoPrint(text.Substring(0, pos));
                    text = text.Substring(pos + 1);
                    pos = text.IndexOf("\n");
                }

            }
            if (text != string.Empty) this.DoPrint(text);
           
            // cleanup exess entries
            while (mobjEntries.Count > mintMaxEntries)
            {
                mobjEntries.RemoveAt(mintMaxEntries - 1); // -1 because of zero-based index
            }
        }

        private void DoPrint(string Text)
        {
            PrintEventArgs with_1 = this.RaiseBeforePrintEvent(Text);
            if (with_1.Cancel == false)
            {
                mobjEntries.Insert(0, with_1.Text);
                this.RaiseAfterPrintEvent(with_1.Text);
            }
        }

        private PrintEventArgs RaiseBeforePrintEvent(string Text)
        {
            PrintEventArgs e = new PrintEventArgs(false, Text);
            try
            {
                if (BeforePrintEvent != null)
                    BeforePrintEvent(this, e);
            }
            catch
            {
            }
            return e;
        }

        private TextEventArgs RaiseAfterPrintEvent(string Text)
        {
            TextEventArgs e = new TextEventArgs(Text);
            try
            {
                if (AfterPrintEvent != null)
                    AfterPrintEvent(this, e);
            }
            catch
            {
            }
            return e;
        }

        public void PrintBlankLine()
        {
            this.DoPrint("");
            while (mobjEntries.Count > mintMaxEntries)
            {
                mobjEntries.RemoveAt(mintMaxEntries - 1); // -1 because of zero-based index
            }
        }

        //Public Sub PrintFormat(ByVal text As String, ByVal params() As Object)
        //    Me.Print(String.Format(text, params))
        //End Sub

        //Public Sub PrintFormat(ByVal text As String, ByVal params As Object)
        //    Me.Print(String.Format(text, params))
        //End Sub

        //Public Sub PrintCharacterToEnTryLine(ByVal character As Char)
        //    If IsVisible = True Then mobjEntryLine.Append(character)
        //End Sub

        //Public Sub PrintCharacterToEnTryLine(ByVal character As String)
        //    If IsVisible = True Then mobjEntryLine.Append(character)
        //End Sub

        public void PrintStringToEnTryLine(string text, bool ProcessEntry, bool Execute)
        {
            if (IsVisible)
            {
                mobjEntryLine.Append(text);
            }
            if (ProcessEntry)
            {
                this.ProcessEntryLine(Execute);
            }
        }

        public void Backspace(int Count)
        {
            for (int idx = 0; idx <= Count - 1; idx++)
            {
                if (mobjEntryLine.Length > 1)
                {
                    mobjEntryLine.Remove(mobjEntryLine.Length - 1, 1);
                }
            }
        }

        //Public Sub ClearEntryLine()
        //    mobjEntryLine = New System.Text.StringBuilder(">")
        //End Sub

        //Public Sub SetEntryLineText(ByVal Text As String)
        //    mobjEntryLine = New System.Text.StringBuilder(">" & Text)
        //End Sub

        public void ProcessEntryLine(bool Execute)
        {
            string text = mobjEntryLine.ToString().Trim(null).Remove(0, 1);
            if (text.Length == 0)
            {
                return;
            }

            if (mintAskingQuestion != 0)
            {
                mstrLastResponceToQuestion = text;
                if (mobjUserResponce != null)
                {
                    mobjUserResponce.Invoke(new string[] { text });
                }
                DoCancelQuestion(false);
            }
            else
            {
                if (mobjCommandManager != null)
                {
                    mobjCommandManager.Buffer.AddLine(text);
                    mobjEntryLine.Remove(1, mobjEntryLine.Length - 1);
                    if (Execute)
                    {
                        mobjCommandManager.Execute(1);
                    }
                }
            }
        }

        public void Render(GameTime gameTime)
        {

            if (IsVisible)
            {

                if (this.mblnAnimateConosle)
                {
                    /// update console state
                    if (mintConsoleState == ConsoleState.Opening && msngCurrentConsolePercentage <= msngMaxConsolePercentage)
                    {
                        if (msngConsoleSpeed <= 0)
                        {
                            msngCurrentConsolePercentage = msngMaxConsolePercentage;
                            mintConsoleState = ConsoleState.Open;
                        }
                        else
                        {

                            msngCurrentConsolePercentage += msngConsoleSpeed * gameTime.ElapsedGameTime.Milliseconds;

                            if (msngCurrentConsolePercentage >= msngMaxConsolePercentage)
                            {

                                msngCurrentConsolePercentage = msngMaxConsolePercentage;
                                mintConsoleState = ConsoleState.Open;

                            }
                        }

                    }
                    else if (mintConsoleState == ConsoleState.Closing && msngCurrentConsolePercentage >= 0.0F)
                    {
                        if (msngConsoleSpeed <= 0)
                        {
                            msngCurrentConsolePercentage = 0.0F;
                            mintConsoleState = ConsoleState.Closed;
                        }
                        else
                        {

                            msngCurrentConsolePercentage -= msngConsoleSpeed * gameTime.ElapsedGameTime.Milliseconds;

                            if (msngCurrentConsolePercentage <= 0.0F)
                            {

                                msngCurrentConsolePercentage = 0.0F;
                                mintConsoleState = ConsoleState.Closed;
                            }
                        }
                    }
                    else
                    {
                        if (mintConsoleState == ConsoleState.Opening)
                        {
                            msngCurrentConsolePercentage = msngMaxConsolePercentage;
                            mintConsoleState = ConsoleState.Open;
                        }
                        else if (mintConsoleState == ConsoleState.Closing)
                        {
                            msngCurrentConsolePercentage = 0.0F;
                            mintConsoleState = ConsoleState.Closed;
                        }
                    }

                }
                else // non animated console
                {


                    if (mintConsoleState == ConsoleState.Opening)
                    {
                        msngCurrentConsolePercentage = msngMaxConsolePercentage;
                        mintConsoleState = ConsoleState.Open;
                    }
                    else if (mintConsoleState == ConsoleState.Closing)
                    {
                        msngCurrentConsolePercentage = 0.0F;
                        mintConsoleState = ConsoleState.Closed;
                    }
                }


                // render console
                //try
                //{
                if (this.mobjRenderConsoleMethod != null)
                {
                    this.mobjRenderConsoleMethod(gameTime);
                }
                //}
                //catch
                //{
                //}

            }
        }

        public void ToggleState()
        {
            if (mintConsoleState == ConsoleState.Closed || mintConsoleState == ConsoleState.Closing)
            {
                mintConsoleState = ConsoleState.Opening;
            }
            else
            {
                this.CancelQuestion();
                mintConsoleState = ConsoleState.Closing;
            }
        }

        /// <summary>
        /// Shows the console.
        /// </summary>
        public void Show()
        {
            if (!this.IsVisible) this.mintConsoleState = ConsoleState.Opening;
            //if (this.IsVisible)
            //{
            //    return;
            //}
            //else
            //{
            //    this.ToggleState();
            //}
        }

        /// <summary>
        /// Hides the console.
        /// </summary>
        public void Hide()
        {
           if (this.IsVisible) this.mintConsoleState = ConsoleState.Closing;
            //if (this.IsVisible == false)
            //{
            //    return;
            //}
            //else
            //{
            //    this.ToggleState();
            //}
        }


        //Public Sub Initialize(ByVal targetControl As System.Windows.Forms.Control)
        //  _targetControl = targetControl

        // create subsystems
        // _renderer = New Renderer(targetControl)

        // initialize subsystems
        //  CommandManager.Instance.Initialize()
        //   Console.Instance.Initialize()
        //End Sub

        //Private Sub Render()
        //    _renderer.BeginScene()

        //    Console.Instance.Render()

        //    _renderer.EndScene()
        //End Sub

        //Public Sub Dispose() Implements System.IDisposable.Dispose
        //   _font.Dispose()
        //  _windowsFont.Dispose()
        // End Sub

        public CommandManager CommandManager
        {
            get
            {
                return mobjCommandManager;
            }
            set
            {
                mobjCommandManager = value;
            }
        }

        //Public ReadOnly Property HasBeenInitialized() As Boolean
        //    Get
        //        Return _HasBeenInit
        //    End Get
        //End Property

        #region " IDisposable Support "
        private bool disposed = false;
        // IDisposable
        private void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    mintAskingQuestion = 0;
                    mobjUserResponce = null;
                    mobjCommandManager = null;
                    this.RenderConsoleMethod = null;
                }
                //  put code to free unmanaged resources here
            }
            this.disposed = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public virtual void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~Console()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(false);
            //base.Finalize();
        }
        #endregion

        public string LastResponceToQuestion
        {
            get { return mstrLastResponceToQuestion; }
        }

        public string CancelString
        {
            get { return mstrCancelString; }
            set { mstrCancelString = value; }
        }

        /// <summary>
        /// Gets/Sets the visibility state of the console.
        /// </summary>
        public ConsoleState State
        {
            get { return mintConsoleState; }
            set { mintConsoleState = value; }
        }

        /// <summary>
        /// Gets/Sets the speed at which the console will open or close.
        /// </summary>
        /// <remarks>
        /// Has no effect if the AnimateConsole property is set to false.
        /// </remarks>
        public float ConsoleSpeed
        {
            get { return msngConsoleSpeed; }
            set { msngConsoleSpeed = value; }
        }
    }

}
