/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;


namespace xGameConsoleXNA
{
	public delegate void LoadExtenderException(Exception e);
	public delegate void ICommandExtenderLoaded(ICommandExtender e);
	
	public class CommandExtenderManager : IDisposable
	{
		
		private System.Collections.Generic.List<ICommandExtender> mobjExtentions;
		private CommandManager mobjCommandManager;
		private bool mblnDisconnectOnDispose = true;

     
        public static List<ICommandExtender> LoadExtenderObjects(string AssemblyFile, LoadExtenderException ExceptionCallback, ICommandExtenderLoaded ExtenderLoadedCallback)
        {
            List<ICommandExtender> Items = new List<ICommandExtender>();
            try
            {
                System.Reflection.Assembly Asm;
                Asm = System.Reflection.Assembly.LoadFrom(AssemblyFile);
                Type[] Types;
                Types = Asm.GetTypes();
                Type T;
                foreach (Type tempLoopVar_T in Types)
                {
                    T = tempLoopVar_T;
                    // make sure type is a public class
                    if (T.IsClass && T.IsPublic)
                    {
                        //check if type implements extender interface
                        if (T.GetInterface(typeof(ICommandExtender).FullName) == null == false)
                        {
                            // make sure type  has constructor that does not have any parameters (simple Sub New() constructor)
                            System.Reflection.ConstructorInfo Con;
                            Con = T.GetConstructor(Type.EmptyTypes);
                            if (Con != null)
                            {
                                // we can now attempt to create the object
                                ICommandExtender Obj = null;
                                try
                                {
                                    Obj = (ICommandExtender)Activator.CreateInstance(T, null);
                                }
                                catch (Exception e)
                                {
                                    try
                                    {
                                        ExceptionCallback.Invoke(e);
                                    }
                                    catch
                                    {
                                    }
                                }

                                if (Obj != null)
                                {
                                    // successfully created object so add item to list
                                    Items.Add(Obj);
                                }

                                try
                                {
                                    ExtenderLoadedCallback.Invoke(Obj);
                                }
                                catch
                                {
                                }
                            }
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                // this exception may be thrown because the assembly may fail to load due to a security exception etc
                try
                {
                    ExceptionCallback.Invoke(ex);
                }
                catch
                {
                }
            }

            //return the list of items loaded
            return Items;
        }
		
		public void AddRange(ICommandExtender[] Ext, bool CallConnect)
		{
			ICommandExtender e;
			foreach (ICommandExtender tempLoopVar_e in Ext)
			{
				e = tempLoopVar_e;
				if (e != null)
				{
					this.Add(e, CallConnect);
				}
			}
		}
		
		public void DisconnectAll()
		{
			ICommandExtender e;
			foreach (ICommandExtender tempLoopVar_e in mobjExtentions)
			{
				e = tempLoopVar_e;
				e.OnDisconnect();
			}
		}
		
		/// <summary>
		/// Adds a object implementing the ICommandExtender interface.
		/// </summary>
		/// <param name="Ext">Any object implementing the ICommandExtender interface.</param>
		/// <param name="CallConnect">If true will call the OnConnect method of the object but only if the IsConnected property is false.</param>
		/// <returns>Returns -1 if the Ext Param was Null (Nothing in vb.net).<br/>
		/// Returns -2 if the OnConnect method threw an exception.<br/>
		/// Will return a value of 0 or more indicating the index location of the added object.</returns>
		/// <remarks></remarks>
		public int Add(ICommandExtender Ext, bool CallConnect)
		{
			if (Ext == null)
			{
				return - 1; // Throw New ArgumentNullException("Ext")
			}
			int Index;
			mobjExtentions.Add(Ext);
			Index = mobjExtentions.IndexOf(Ext);
			if (Ext.IsConnected == false && CallConnect)
			{
				try
				{
					Ext.OnConnect(mobjCommandManager, this);
				}
				catch
				{
					mobjExtentions.RemoveAt(Index);
					return - 2;
				}
			}
			return Index;
		}
		
		public int Count
		{
			get
			{
				return mobjExtentions.Count;
			}
		}
		
		public ICommandExtender GetExtender(int Index)
		{
			if (Index < 0 || Index > mobjExtentions.Count - 1)
			{
				throw (new ArgumentOutOfRangeException("Index"));
			}
			
			return ((ICommandExtender) mobjExtentions[Index]);
		}
		
		public void Remove(int Index, bool CallDisconnect)
		{
			if (Index < 0 || Index > mobjExtentions.Count - 1)
			{
				throw (new ArgumentOutOfRangeException("Index"));
			}
			if (CallDisconnect && mobjExtentions[Index].IsConnected)
			{
				mobjExtentions[Index].OnDisconnect();
			}
			mobjExtentions.RemoveAt(Index);
		}
		
		public void Remove(ICommandExtender Extention, bool CallDisconnect)
		{
			int Index;
			Index = this.mobjExtentions.IndexOf(Extention);
			if (Index < 0 || Index > mobjExtentions.Count - 1)
			{
				throw (new ArgumentOutOfRangeException("Index"));
			}
			
			if (CallDisconnect && mobjExtentions[Index].IsConnected)
			{
				mobjExtentions[Index].OnDisconnect();
			}
			mobjExtentions.RemoveAt(Index);
		}
		
		public CommandExtenderManager(CommandManager CommandReference)
		{
			mobjCommandManager = CommandReference;
			mobjExtentions = new System.Collections.Generic.List<ICommandExtender>();
		}
		
		#region " IDisposable Support "
		private bool disposed = false;
		// IDisposable
		private void Dispose(bool disposing)
		{
			if (! this.disposed)
			{
				if (disposing)
				{
					if (mblnDisconnectOnDispose)
					{
						this.DisconnectAll();
					}
					
					mobjExtentions.Clear();
					mobjExtentions = null;
					mobjCommandManager = null;
				}
				// put code to free unmanaged resources here
			}
			this.disposed = true;
		}
		
		// This code added by Visual Basic to correctly implement the disposable pattern.
		public virtual void Dispose()
		{
			// Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
			Dispose(true);
			GC.SuppressFinalize(this);
		}
		
		~CommandExtenderManager()
		{
			// Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
			Dispose(false);
			//base.Finalize();
		}
		#endregion
		
		public CommandManager CommandManager
		{
			get
			{
				return mobjCommandManager;
			}
			set
			{
				mobjCommandManager = value;
			}
		}
		
		public bool CallDisconnectOnDispose
		{
			get
			{
				return mblnDisconnectOnDispose;
			}
			set
			{
				mblnDisconnectOnDispose = value;
			}
		}
	}
	
}
