/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using System.Reflection;
#endregion

namespace xGameConsoleXNA
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class ConsoleComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        protected internal VertexBuffer vb;
        private Color backColor = new Color(0, 0, 0, 128);
        protected internal BasicEffect fx;
        private Rectangle bounds = new Rectangle(0, 0, 600, 400);
        protected internal Nuclex.Fonts.BitmapFont font;
        private Console console;
        private RenderConsole rendermethod;
        private CommandManager commandManager;
        private KeyboardState prevstate;
        private VertexDeclaration decl;
        private Keys displayToggle = Keys.OemTilde;
        long RenderConsole_LastCursonBlinkTime;
        bool RenderConsole_CursorVisible;
        private Color borderColor = Color.White;
        private Color textColor = Color.White;
        private CommandExtenderManager extenderManager;
        internal ContentManager content;
        private int lineSpacing = 4;

           
        
        public void Print(string[] text)
        {
            this.console.Print(text); 
        }

        public void Print(System.Collections.Specialized.StringCollection text)
        {
            this.console.Print(text);          
        }

        public void Print(string text)
        {
            this.console.Print(text);
        }

        public int LineSpacing
        {
            get { return lineSpacing; }
            set { lineSpacing = value; }
        }

        public Nuclex.Fonts.BitmapFont Font
        {
            get { return this.font; }
            set { this.font = value; }
        }

        public CommandExtenderManager ExtenderManager
        {
            get { return extenderManager; }
        }

        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        public Color BorderColor
        {
            get { return borderColor; }
            set { borderColor = value; }
        }

        public Keys DisplayToggle
        {
            get { return displayToggle; }
            set { displayToggle = value; }
        }

        public CommandManager CommandManager
        {
            get { return commandManager; }
        }

        public RenderConsole RenderMethod
        {
            get { return rendermethod; }
            set { rendermethod = value; }
        }

        public Console Console
        {
            get { return console; }
        }

        public Rectangle Bounds
        {
            get { return bounds; }
            set { bounds = value; }
        }

        public int Left
        {
            get { return bounds.X; }
            set { bounds.X = value; }
        }

        public int Top
        {
            get { return bounds.Y; }
            set { bounds.Y = value; }
        }

        public int Width
        {
            get { return bounds.Width; }
            set { bounds.Width = value; }
        }

        public int Height
        {
            get { return bounds.Height; }
            set { bounds.Height = value; }
        }

        public Color BackgroundColor
        {
            get { return backColor; }
            set
            {
                backColor = value;
                CreateVB(backColor);
            }
        }

        public ConsoleComponent(Game game)
            : base(game)
        {
            commandManager = new CommandManager();
            console = new Console(this.commandManager);
            console.RenderConsoleMethod = this.Render;
            this.extenderManager = new CommandExtenderManager(this.commandManager);
            content = new ContentManager(game.Services);
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();

            // TODO: Add your initialization code here
            CreateVB(backColor);
            if (decl == null) decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionNormalTexture.VertexElements);

            //Assembly assembly = Assembly.GetExecutingAssembly();

            //// load default font
            //string file = "xGameConsoleXNA.Resources.Console.dds.xml";
            //if (font != null) font.Dispose();
            //font = null;
            //font = XNAFont.LoadFont(this.GraphicsDevice, assembly.GetManifestResourceStream(file), string.Empty);
            //font.LineSpacing = 2;
            //this.console.LineHeight = font.Height + font.LineSpacing;

            //// load font texture
            //if (font.Texture != null && font.Texture.IsDisposed == false) font.Texture.Dispose();
            //font.Texture = null;
            //file = "xGameConsoleXNA.Resources.Console.dds";
            //font.Texture = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
            //if (font != null) font.Dispose();
            //font = null;
            //font = this.content.Load<Nuclex.Fonts.BitmapFont>("Resources\\DefaultFont");
            //this.console.LineHeight = font.MeasureString("jT").Height + 5;

            if (fx != null && fx.IsDisposed == false) fx.Dispose();
            fx = null;
            fx = new BasicEffect(this.GraphicsDevice, null);

            this.Visible = false;
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {

            if (!this.Visible) return;
            KeyboardState state = Keyboard.GetState();

            Keys[] keys = prevstate.GetPressedKeys();
            foreach (Keys k in keys)
            {
                // check for key click
                if (state.IsKeyUp(k))
                {
                    switch (k)
                    {
                        case Keys.Enter:
                            console.ProcessEntryLine(true);
                            break;

                        case Keys.Execute:
                            console.ProcessEntryLine(true);
                            break;

                        case Keys.Back:
                            this.console.Backspace(1);
                            break;

                        case Keys.Escape:
                            // check if showing or hiding console
                            if (this.console.IsVisible)
                            {
                                this.console.Hide();
                                prevstate = state;
                                return;
                            }
                            //else
                            //{
                            //    this.console.Show();
                            //}
                            break;


                        case Keys.Apps:
                        case Keys.Attn:
                        case Keys.BrowserBack:
                        case Keys.BrowserFavorites:
                        case Keys.BrowserForward:
                        case Keys.BrowserHome:
                        case Keys.BrowserRefresh:
                        case Keys.BrowserSearch:
                        case Keys.BrowserStop:
                        case Keys.CapsLock:
                        case Keys.Delete:
                        case Keys.Down:
                        case Keys.End:
                        case Keys.EraseEof:
                        case Keys.Exsel:
                        case Keys.F1:
                        case Keys.F2:
                        case Keys.F3:
                        case Keys.F4:
                        case Keys.F5:
                        case Keys.F6:
                        case Keys.F7:
                        case Keys.F8:
                        case Keys.F9:
                        case Keys.F10:
                        case Keys.F11:
                        case Keys.F12:
                        case Keys.F13:
                        case Keys.F14:
                        case Keys.F15:
                        case Keys.F16:
                        case Keys.F17:
                        case Keys.F18:
                        case Keys.F19:
                        case Keys.F20:
                        case Keys.F21:
                        case Keys.F22:
                        case Keys.F23:
                        case Keys.F24:
                        case Keys.Help:
                        case Keys.Home:
                        case Keys.Insert:
                        case Keys.LaunchApplication1:
                        case Keys.LaunchApplication2:
                        case Keys.LaunchMail:
                        case Keys.Left:
                        case Keys.LeftAlt:
                        case Keys.LeftControl:
                        case Keys.LeftShift:
                        case Keys.LeftWindows:
                        case Keys.MediaNextTrack:
                        case Keys.MediaPlayPause:
                        case Keys.MediaPreviousTrack:
                        case Keys.MediaStop:
                        case Keys.None:
                        case Keys.NumLock:
                        case Keys.Oem8:
                        case Keys.OemClear:
                        case Keys.Pa1:
                        case Keys.PageDown:
                        case Keys.PageUp:
                        case Keys.Play:
                        case Keys.Print:
                        case Keys.PrintScreen:
                        case Keys.ProcessKey:
                        case Keys.Right:
                        case Keys.RightAlt:
                        case Keys.RightControl:
                        case Keys.RightShift:
                        case Keys.RightWindows:
                        case Keys.Scroll:
                        case Keys.Select:
                        case Keys.SelectMedia:
                        case Keys.Sleep:
                        case Keys.Up:
                        case Keys.VolumeDown:
                        case Keys.VolumeMute:
                        case Keys.VolumeUp:
                        case Keys.Zoom:
                            break;

                        case Keys.Add:
                            console.PrintStringToEnTryLine("+", false, false);
                            break;

                        case Keys.Subtract:
                            console.PrintStringToEnTryLine("-", false, false);
                            break;

                        case Keys.Multiply:
                            console.PrintStringToEnTryLine("*", false, false);
                            break;

                        case Keys.OemOpenBrackets:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("[", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("{", false, false);
                            }
                            break;

                        case Keys.OemCloseBrackets:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("]", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("}", false, false);
                            }
                            break;

                        case Keys.OemComma:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine(",", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("<", false, false);
                            }
                            break;

                        case Keys.OemMinus:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("-", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("_", false, false);
                            }
                            break;

                        case Keys.OemPeriod:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine(".", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine(">", false, false);
                            }
                            break;

                        case Keys.OemPipe:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine(@"\", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("|", false, false);
                            }
                            break;

                        case Keys.OemPlus:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("=", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("+", false, false);
                            }
                            break;

                        case Keys.OemTilde:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("`", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("~", false, false);
                            }
                            break;

                        case Keys.Tab:
                            console.PrintStringToEnTryLine("    ", false, false);
                            break;

                        case Keys.NumPad0:
                            console.PrintStringToEnTryLine("0", false, false);
                            break;

                        case Keys.NumPad1:
                            console.PrintStringToEnTryLine("1", false, false);
                            break;

                        case Keys.NumPad2:
                            console.PrintStringToEnTryLine("2", false, false);
                            break;

                        case Keys.NumPad3:
                            console.PrintStringToEnTryLine("3", false, false);
                            break;

                        case Keys.NumPad4:
                            console.PrintStringToEnTryLine("4", false, false);
                            break;

                        case Keys.NumPad5:
                            console.PrintStringToEnTryLine("5", false, false);
                            break;

                        case Keys.NumPad6:
                            console.PrintStringToEnTryLine("6", false, false);
                            break;

                        case Keys.NumPad7:
                            console.PrintStringToEnTryLine("7", false, false);
                            break;

                        case Keys.NumPad8:
                            console.PrintStringToEnTryLine("8", false, false);
                            break;

                        case Keys.NumPad9:
                            console.PrintStringToEnTryLine("9", false, false);
                            break;

                        case Keys.OemQuotes:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("'", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("\"", false, false);
                            }
                            break;

                        case Keys.OemQuestion:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("/", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("?", false, false);
                            }
                            break;

                        case Keys.OemBackslash:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine(@"\", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("|", false, false);
                            }
                            break;

                        case Keys.OemSemicolon:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine(";", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine(":", false, false);
                            }
                            break;

                        case Keys.Separator:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("<", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine(",", false, false);
                            }
                            break;

                        case Keys.Divide:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("/", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("?", false, false);
                            }
                            break;

                        case Keys.Decimal:
                            console.PrintStringToEnTryLine(".", false, false);
                            break;

                        case Keys.Space:
                            console.PrintStringToEnTryLine(" ", false, false);
                            break;

                        case Keys.D1:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("1", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("!", false, false);
                            }
                            break;

                        case Keys.D2:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("2", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("@", false, false);
                            }
                            break;

                        case Keys.D3:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("3", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("#", false, false);
                            }
                            break;

                        case Keys.D4:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("4", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("$", false, false);
                            }
                            break;

                        case Keys.D5:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("5", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("%", false, false);
                            }
                            break;

                        case Keys.D6:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("6", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("^", false, false);
                            }
                            break;

                        case Keys.D7:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("7", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("&", false, false);
                            }
                            break;

                        case Keys.D8:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("8", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("*", false, false);
                            }
                            break;

                        case Keys.D9:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("9", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine("(", false, false);
                            }
                            break;

                        case Keys.D0:
                            if (!state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                console.PrintStringToEnTryLine("0", false, false);
                            }
                            else
                            {
                                console.PrintStringToEnTryLine(")", false, false);
                            }
                            break;

                        default:
                            string c;
                            c = k.ToString();
                            if (!state.IsKeyDown(Keys.CapsLock) && !state.IsKeyDown(Keys.LeftShift) && !state.IsKeyDown(Keys.RightShift))
                            {
                                c = c.ToLower();
                            }
                            console.PrintStringToEnTryLine(c, false, false);

                            break;
                    }
                }
            }

            prevstate = state;
            base.Update(gameTime);
        }

        public void Show()
        {
            this.Visible = true;
            this.console.Show();
        }

        public void Hide()
        {
            this.Visible = false;
            this.console.Hide();
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {

            this.console.Render(gameTime);
            if (console.IsVisible == false)
            {
                this.Visible = false;
                return;
            }

            base.Draw(gameTime);
        }

        private void Render(GameTime gameTime)
        {
            xGameConsoleXNA.Properties.Settings sett = xGameConsoleXNA.Properties.Settings.Default;

            if (gameTime.TotalGameTime.Ticks > RenderConsole_LastCursonBlinkTime + TimeSpan.FromMilliseconds(sett.CursorBlinkRate).Ticks)
            {
                RenderConsole_CursorVisible = !RenderConsole_CursorVisible;
                RenderConsole_LastCursonBlinkTime = gameTime.TotalGameTime.Ticks;
            }

            int consoleHeight = (int)(this.bounds.Height * (this.console.MaxConsolePercentage / 100));
            int visibleHeight = (int)(this.bounds.Height * (this.console.CurrentConsolePercentage / 100));

            int x = this.bounds.X;
            int y = this.bounds.Y;
            int LineHeight = this.font.MeasureString(">jT").Height + this.lineSpacing;
            int line = y + (visibleHeight - LineHeight );
            int width = this.bounds.Width;
            //  Dim height As Single = visibleHeight
            Rectangle consoleBackground = new Rectangle(x, y, width, visibleHeight);

            DrawRect(consoleBackground, true, this.backColor);

            //consoleBackground.Inflat(1, 1)
            DrawRect(consoleBackground, false, this.borderColor);
            DrawRect(new Rectangle(consoleBackground.Left, consoleBackground.Bottom - LineHeight,
                     consoleBackground.Width, 0), false, this.borderColor);

            if (this.font == null) return;

            // hold the current state of the sissor rect
            Rectangle sr = this.GraphicsDevice.ScissorRectangle;
            // set the sissor rect to the console dimentions so text does not get drawn out of bounds
            Rectangle nsr;
            nsr = Helpers.RectangleIntersect(Helpers.ViewportRectangle(this.GraphicsDevice.Viewport), consoleBackground);
            if (nsr == Rectangle.Empty)
            {
                this.GraphicsDevice.ScissorRectangle = consoleBackground;
            }
            else
            {
                this.GraphicsDevice.ScissorRectangle = nsr;
            }

            // hold the state of the sissor test
            bool sTest = this.GraphicsDevice.RenderState.ScissorTestEnable;
            // enable sissor testing
            this.GraphicsDevice.RenderState.ScissorTestEnable = true;

            consoleBackground.Inflate(-4, 0);
            // draw entry line text with the blinking cursor at the end
            if (RenderConsole_CursorVisible)
            {
                font.DrawString(new Vector2(consoleBackground.Left, line), this.console.Entryline + sett.CursorBlickChar,
                                Nuclex.Fonts.TextAlignment.Left | Nuclex.Fonts.TextAlignment.Top, this.textColor);
            }
            else
            {
                font.DrawString(new Vector2(consoleBackground.Left, line), this.console.Entryline,
                                Nuclex.Fonts.TextAlignment.Left | Nuclex.Fonts.TextAlignment.Top, this.textColor);
            }

            // calc next line pos. This will be the start of the Console.Entries.
            line -= LineHeight;

            // draw stored entries
            string entry;
            foreach (string tempLoopVar_entry in this.console.Entries)
            {
                entry = tempLoopVar_entry;
                if (line <= -LineHeight) break;
               
                font.DrawString(new Vector2(consoleBackground.Left, line), entry,
                                Nuclex.Fonts.TextAlignment.Left | Nuclex.Fonts.TextAlignment.Top, this.textColor);
                line -= LineHeight;
            }

            // restore sissor rect and test
            this.GraphicsDevice.ScissorRectangle = sr;
            this.GraphicsDevice.RenderState.ScissorTestEnable = sTest;
        }

        private void DrawRect(Rectangle r, bool fill, Color col)
        {
            // setup orthographic projection with same bounds as graphics devices viewport
            fx.Projection = Matrix.CreateOrthographicOffCenter(this.GraphicsDevice.Viewport.X,
                                                               this.GraphicsDevice.Viewport.X + this.GraphicsDevice.Viewport.Width,
                                                               this.GraphicsDevice.Viewport.Y + this.GraphicsDevice.Viewport.Height,
                                                               this.GraphicsDevice.Viewport.Y, 1, 10000);
            fx.View = Matrix.CreateLookAt(Vector3.Backward, Vector3.Zero, Vector3.Up);
            fx.World = Matrix.CreateScale(r.Width, r.Height, 1);
            fx.World *= Matrix.CreateTranslation(r.X, r.Y, 0);
            fx.VertexColorEnabled = false;
            fx.TextureEnabled = false;
            fx.LightingEnabled = false;
            fx.DiffuseColor = col.ToVector3();
            fx.Alpha = col.A / (float)byte.MaxValue;
            //fx.DirectionalLight0.Direction = Vector3.Forward;
            //fx.DirectionalLight0.Enabled = true;
            //fx.DirectionalLight0.DiffuseColor = col.ToVector3();
            fx.Texture = null;
            fx.Begin();
            for (int idx = 0; idx <= fx.CurrentTechnique.Passes.Count - 1; idx++)
            {
                EffectPass pass;
                pass = fx.CurrentTechnique.Passes[idx];
                pass.Begin();
                this.GraphicsDevice.VertexDeclaration = decl;
                this.GraphicsDevice.Vertices[0].SetSource(vb, 0, VertexPositionNormalTexture.SizeInBytes);
                if (fill)
                {
                    this.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;
                    this.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleFan, 0, 2);
                }
                else
                {
                    this.GraphicsDevice.DrawPrimitives(PrimitiveType.LineStrip, 0, 4);
                }
                pass.End();
            }
            fx.End();
        }

        /// <summary>
        /// Creates a triangle fan of 2 polys along the x/y axis starting at 0,0,0 with a width and height of 1.
        /// </summary>
        /// <returns>Returns a new vertexbuffer</returns>
        private void CreateVB(Color col)
        {
            VertexPositionNormalTexture[] verts = new VertexPositionNormalTexture[5];
            //verts[0] = new VertexPositionColor(new Vector3(0, 0, 0), col);
            //verts[1] = new VertexPositionColor(new Vector3(0, 1, 0), col);
            //verts[2] = new VertexPositionColor(new Vector3(1, 1, 0), col);
            //verts[3] = new VertexPositionColor(new Vector3(1, 0, 0), col);
            //verts[4] = verts[0];

            verts[0] = new VertexPositionNormalTexture(new Vector3(0, 0, 0), new Vector3(0, 0, 0) + Vector3.Backward, Vector2.Zero);
            verts[1] = new VertexPositionNormalTexture(new Vector3(0, 1, 0), new Vector3(0, 1, 0) + Vector3.Backward, Vector2.UnitY);
            verts[2] = new VertexPositionNormalTexture(new Vector3(1, 1, 0), new Vector3(1, 1, 0) + Vector3.Backward, Vector2.UnitX + Vector2.UnitY);
            verts[3] = new VertexPositionNormalTexture(new Vector3(1, 0, 0), new Vector3(1, 0, 0) + Vector3.Backward, Vector2.UnitX);
            verts[4] = verts[0];

            if (vb != null && vb.IsDisposed == false) vb.Dispose();
            vb = null;
            vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionNormalTexture.SizeInBytes,
                                  ResourceUsage.None, ResourceManagementMode.Automatic);
            vb.SetData<VertexPositionNormalTexture>(verts);
            return;
        }

    }
}


