/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xGameConsoleXNA
{
    public sealed class Helpers
    {
        public static T GetAttribute<T>(System.Reflection.MemberInfo m) where T : Attribute
        {
            // get command attributes
            object[] items = m.GetCustomAttributes(false);
            foreach (object att in items)
            {
                // check if the VisibleToUsers was specified
                if (att is T) return (T)att;

            }

            return null;
        }

        public static int FindLongestString(string[] items)
        {
            int count = 0;
            foreach (string s in items)
            {
                if (count < s.Length) count = s.Length;
            }
            return count;
        }


        // This rectangle intersect code brout to you by Mono! ... Well sort of...
        // Code was edited to include IntersectsWithInclusive and FromLTRB methods code
        // Get the Modo Code here http://go-mono.com/sources-stable/
        public static Rectangle RectangleIntersect(Rectangle r1, Rectangle r2)
        {
            // MS.NET returns a non-empty rectangle if the two rectangles
            // touch each other
            if (!((r1.Left > r2.Right) || (r1.Right < r2.Left) ||
                 (r1.Top > r2.Bottom) || (r1.Bottom < r2.Top)))
                return Rectangle.Empty;

            int L, T, R, B;
            L = Math.Max(r1.Left, r2.Left);
            T = Math.Max(r1.Top, r2.Top);
            R = Math.Min(r1.Right, r2.Right);
            B = Math.Min(r1.Bottom, r2.Bottom);
            return new Rectangle(L, T, R - L, B - T);
        }

        public static Rectangle ViewportRectangle(Viewport vp)
        {
            return new Rectangle(vp.X, vp.Y, vp.Width, vp.Height);
        }

        public static string Space(int count)
        {
            return new string(' ', count);
        }

        public static string String(char c, int count)
        {
            return new string(c, count);
        }

        public static bool IsNumeric(string value)
        {
            value = value.Trim();
            if (value == string.Empty) return false;
            if (char.IsSymbol(value[0]) == false && char.IsNumber(value[0]) == false) return false;
            if (char.IsSymbol(value[0]) && value.Length > 1 && char.IsNumber(value[1]) == false) return false;
            return true;
        }

        public static string StringArrayToString(System.Collections.Specialized.StringCollection Values, int StartIndex, char Separator)
        {
            return StringArrayToString(Values, StartIndex, Values.Count, Separator);
        }

        public static string StringArrayToString(System.Collections.Specialized.StringCollection Values, int StartIndex, int Count, char Separator)
        {
            if (Values == null)
            {
                return string.Empty;
            }
            if (Values.Count == 0)
            {
                return string.Empty;
            }

            string Tmp = string.Empty;

            for (int idx = StartIndex; idx <= StartIndex + Count - 1; idx++)
            {
                if (idx > Values.Count - 1)
                {
                    break;
                }
                Tmp += Separator + Values[idx];
            }

            Tmp = Tmp.TrimStart(Separator);
            return Tmp;
        }

        public static string StringCollectionToString(System.Collections.Specialized.StringCollection SC, string Separator)
        {
            string[] TmpArgs = null;
            TmpArgs = new string[SC.Count - 1 + 1];
            SC.CopyTo(TmpArgs, 0);
            return string.Join(Separator, TmpArgs);
        }

        public static string[] StringCollectionToArray(System.Collections.Specialized.StringCollection SC)
        {
            string[] TmpArgs = null;
            TmpArgs = new string[SC.Count - 1 + 1];
            SC.CopyTo(TmpArgs, 0);
            return TmpArgs;
        }

        public static string StringArrayToString(string[] Values, int StartIndex, int Count, char Separator)
        {
            if (Values == null)
            {
                return string.Empty;
            }
            if (Values.Length == 0)
            {
                return string.Empty;
            }

            string Tmp = string.Empty;

            for (int idx = StartIndex; idx <= StartIndex + Count - 1; idx++)
            {
                if (idx > Values.Length - 1)
                {
                    break;
                }
                Tmp += Separator + Values[idx];
            }

            Tmp = Tmp.TrimStart(Separator);
            return Tmp;
        }

        public static string StringArrayToString(string[] Values, int StartIndex, char Separator)
        {
            return StringArrayToString(Values, StartIndex, Values.Length, Separator);
        }
    }

}
